; =======================================================================
FUNCTION ccf_read_ccf, $
   ccf_name, $                   ; CCF name requested
   tsd, $                        ; to check the time range
   TEST=test, $                  ; Force to read TEST version CCF
   SAFE=safe, $                  ; Force to read SAFE version CCF
   CCF_FILE=ccf_file, $          ; Specify CCF file name directly
   CCF_CNTL_FILE=ccf_cntl_file, $; Specify control file name (for test)
   HEADER=hdr, $                 ; (returned) CCF Header information
   ERROR=error                   ; (returned) Error status (!SUCCEED/!FAIL)

; 06/01/18 I.Yamamura version 1.2
; 06/05/13 I.Yamamura version 1.3
; 07/01/17 I.Yamamura recognize AOT mode in FITS header.

; reset error
   error = !SUCCEED

; Control File Name
   CCF_CNTL_FILE_NAME = 'CNTL__001_001.ccf'

; Select apropriate CCF file

   if (NOT Keyword_Set(CCF_FILE)) then begin

      if (n_params() LT 1) then begin
         lmessage, 1, 'Give A CCF_Name!'
         error = !FAIL
         return, error
      endif

      if (NOT Keyword_Set(CCF_CNTL_FILE)) then $
         ccf_cntl_file = !CCF_PATH+'/'+CCF_CNTL_FILE_NAME

      cntl = CCF_Read_CCF_File(ccf_cntl_file, HEADER=chdr, ERROR=err)
      if (err EQ !FAIL) then begin
         lmessage, 1, 'Invalid CCF CNTL File.'
         error = !FAIL
         return, error
      endif

;;;      help, cntl, /struct

; select requested CCF_NAME
      ccf_name = STRUPCASE(ccf_name)
      c = cntl.NAME
      idx = WHERE (c EQ ccf_name, cnt)
      if (cnt EQ 0) then begin
         lmessage, 1, 'Invalid NAME ('+ccf_name+').'
         error = !FAIL
         return, error
      endif
      cntl = cntl[idx]

; select by AOT
      pheader = tsd->Get_Header_TSD('PRIMARY')
      idx = WHERE(STRPOS(pheader, 'AOT     ') GE 0, cnt)
      if (cnt EQ 0) then begin
         lmessage, 3, 'No AOT information in FITS Header. Assumeing SURVEY.'
         c = cntl.FISAS
      endif else begin
         if (cnt GE 2) then begin
            lmessage, 1, 'Multiple AOT information in FITS Header.'
            error = !FAIL
            return, error
         endif
         aothd = pheader[idx]
         if (STRPOS(aothd, 'SURVEY') GE 0) then begin
            c = cntl.FISAS
         endif else if (STRPOS(aothd, "'        '") GE 0) then begin
            lmessage, 5, 'Empty AOT information in FITS Header. Assumeing SURVEY.'
            c = cntl.FISAS
         endif else if (STRPOS(aothd, 'FIS01') GE 0) then begin
            c = cntl.FIS01
         endif else if (STRPOS(aothd, 'FIS02') GE 0) then begin
            c = cntl.FIS02
         endif else if (STRPOS(aothd, 'FIS03') GE 0) then begin
            c = cntl.FIS03
         endif else begin
            msg = STRING(FORMAT='(A,A,A)', 'Unknown AOT: ', aothd, '.')
            lmessage, 1, msg
            error = !FAIL
            return, error
         endelse
      endelse

      idx = WHERE(c NE 0, cnt)
      if (cnt EQ 0) then begin
         lmessage, 1, 'No appropriate CCF for this AOT ('+STRMID(aothd, 10, 14)+').'
         error = !FAIL
         return, error
      endif
      cntl = cntl[idx]

; select one with TEST or SAFE status
; SAFE overidges TEST if both options are given.
      if (Keyword_Set(TEST) OR (!CCF_MODE EQ 'TEST')) then begin
         c = cntl.TEST
      endif else if (Keyword_Set(SAFE) OR (!CCF_MODE EQ 'SAFE')) then begin
         c = cntl.SAFE
      endif else begin
         lmessage, 1, 'Invalid !CCF_MODE ('+mode+')'
         error = !FAIL
         return, error
      end

      idx = WHERE(c EQ !TRUE, cnt)
      if (cnt EQ 0) then begin
         lmessage, 1, 'No Valid CCF.'
         error = !FAIL
         return, error
      endif
      cntl = cntl[idx]

; skip selection by time
;;   idx = WHERE ((aftime GE cntl.valid_from) AND (aftime LE cntl.valid_to), cnt)
;;   if (cnt EQ 0) then begin
;;      lmessage, 1, 'Invalid AFTIME ('+STRING(FORMAT='(F10)', aftime)+').'
;;      return = -1
;;   endif else if (cnt GE 2) then begin
;;      lmessage, 1, 'Valid CCF for this aftime ('+STRING(FORMAT='(F10)', aftime)+') is not unique ('+STRING(cntl.ccf_filename, /PRINT)+').'
;;      lmessage, 1, 'Check CCF control table ('+ccf_ctrl_file+').'
;;      error = !FAIL
;;      return, error
;;   endif
;;
;;   cntl = cntl[idx]

      if (cnt EQ 0) then begin
         lmessage, 1, 'No Valid CCF.'
         error = !FAIL
         return, error
      endif

; this is temporary
      if (cnt GT 1) then begin
         lmessage, 1, 'More than two files found. Take the 1st one.'
         error = !FAIL
         return, error
      endif
      cntl = cntl[0]

; absolute CCF file name construction
      name = cntl.NAME
      if (STRLEN(name) EQ 4) then name = name + '_'
      ccf_file = STRING(FORMAT='(A, "/", A, "_", I3.3, "_", I3.3, ".ccf")', $
         !CCF_PATH, name, cntl.SET_NO, cntl.REV_NO)
   end

; read CCF file
   ccf = CCF_Read_CCF_File(ccf_file, HEADER=hdr, ERROR=error)

;;;   help, ccf

   RETURN, ccf
END
