; $Id: list_vdet.pro,v 1.11 2006/03/23 06:23:01 baba Exp $
;+
; NAME:
;       LIST_VDET
;
; PURPOSE:
;       The purpose of this procedure is to show the list of
;       detector signals. This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.ac.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       list_vdet, GROUP_LEADER=tlb
;
; OTHER PROGRAMS NEEDED:
;
;   The following programs are required to run this program:
;    - list_vdet_ps.pro
;
; MODIFICATION HISTORY:
;        Written by Makiuti,S.
;        06/03/08 Last modified
;-

;-----------------------------------------------------------------
pro CloseLV, Event
  WIDGET_CONTROL, Event.top, /DESTROY
  return
end

;-----------------------------------------------------------------
pro DrawLV, Event
COMMON Share
;COMMON Share_test, tsd ;For test

  ; Get Widget IDs
  WID_TEXT_Xmin = widget_info(Event.top, find_by_uname='Xmin')
  WID_TEXT_Xmax = widget_info(Event.top, find_by_uname='Xmax')
  WID_TEXT_Ymin = widget_info(Event.top, find_by_uname='Ymin')
  WID_TEXT_Ymax = widget_info(Event.top, find_by_uname='Ymax')
  WID_DataType = widget_info(Event.top, find_by_uname='DataType')

  ; Get values
  WIDGET_CONTROL, WID_TEXT_Xmin, GET_VALUE=xmin
  WIDGET_CONTROL, WID_TEXT_Xmax, GET_VALUE=xmax
  WIDGET_CONTROL, WID_TEXT_Ymin, GET_VALUE=ymin
  WIDGET_CONTROL, WID_TEXT_Ymax, GET_VALUE=ymax
  WIDGET_CONTROL, WID_DataType, GET_VALUE=value

  ; Data values to draw
  det_data = (value eq 0) ? tsd->get_value_tsd('FIS_OBS', 'DET'):tsd->get_value_tsd('FIS_OBS', 'FLUX')
  x = indgen(n_elements(det_data[0,*]), /L64)
  vdet_ch_num = n_elements(det_data[*,0])

  xrange = (n_elements(float(xmin)) NE 0) AND (n_elements(float(xmax)) NE 0) ? $
           [float(xmin), float(xmax)] : [0,0]
  yrange = (n_elements(float(ymin)) NE 0) AND (n_elements(float(ymax)) NE 0) ? $
           [float(ymin), float(ymax)] : [0,0]

  ; Set the window for drawing
  DrawWinID = widget_info(Event.top, find_by_uname='DrawWin')
  Widget_Control, DrawWinID, Get_Value=drawIndex
  wset, drawIndex

  ; Draw det_data list
  !p.multi=[0,vdet_ch_num/5,5,0,0]
  FOR i = 0, vdet_ch_num-1, 1 DO BEGIN

    ch_num=i+1

    IF (vdet_ch_num EQ 100) THEN $
      ch = 'SW Detector ch'+string(ch_num, format='(i3)') ;pixel to plot
    IF (vdet_ch_num EQ 75) THEN $
      ch = 'LW Detector ch'+string(ch_num, format='(i3)') ;pixel to plot
    ; Plot
    plot, x, det_data[i,*], TITLE=ch, charsize=1.4, /ynozero, $
        xticks=2, XTITLE='Index', $
        xrange=xrange, yrange=yrange, xstyle=1, ystyle=1, $
        background=getcolor('Black', !D.Table_Size-2), color=getcolor('Light Cyan', !D.Table_Size-3)

  ENDFOR
  !p.multi=0

  return
end

;-----------------------------------------------------------------
pro ScrLeftLV, Event

  ; Get Widget IDs
  WID_TEXT_Xmin = widget_info(Event.top, find_by_uname='Xmin')
  WID_TEXT_Xmax = widget_info(Event.top, find_by_uname='Xmax')

  ; Get values
  WIDGET_CONTROL, WID_TEXT_Xmin, GET_VALUE=xmin
  WIDGET_CONTROL, WID_TEXT_Xmax, GET_VALUE=xmax

  ; Set new X range
  range = float(xmax) - float(xmin)
  xmax = string(float(xmin))
  xmin = string(float(xmin) - range)
  WIDGET_CONTROL, WID_TEXT_Xmin, SET_VALUE=xmin
  WIDGET_CONTROL, WID_TEXT_Xmax, SET_VALUE=xmax

end

;-----------------------------------------------------------------
pro ScrRightLV, Event

  ; Get Widget IDs
  WID_TEXT_Xmin = widget_info(Event.top, find_by_uname='Xmin')
  WID_TEXT_Xmax = widget_info(Event.top, find_by_uname='Xmax')

  ; Get values
  WIDGET_CONTROL, WID_TEXT_Xmin, GET_VALUE=xmin
  WIDGET_CONTROL, WID_TEXT_Xmax, GET_VALUE=xmax

  ; Set new X range
  range = float(xmax) - float(xmin)
  xmin = string(float(xmax))
  xmax = string(float(xmax) + range)
  WIDGET_CONTROL, WID_TEXT_Xmin, SET_VALUE=xmin
  WIDGET_CONTROL, WID_TEXT_Xmax, SET_VALUE=xmax

end

;-----------------------------------------------------------------
pro PsLV, Event
COMMON Share
;COMMON Share_test, tsd ;For test

  ; Get Widget IDs
  WID_TEXT_Xmin = widget_info(Event.top, find_by_uname='Xmin')
  WID_TEXT_Xmax = widget_info(Event.top, find_by_uname='Xmax')
  WID_TEXT_Ymin = widget_info(Event.top, find_by_uname='Ymin')
  WID_TEXT_Ymax = widget_info(Event.top, find_by_uname='Ymax')
  WID_DataType = widget_info(Event.top, find_by_uname='DataType')

  ; Get values
  WIDGET_CONTROL, WID_TEXT_Xmin, GET_VALUE=xmin
  WIDGET_CONTROL, WID_TEXT_Xmax, GET_VALUE=xmax
  WIDGET_CONTROL, WID_TEXT_Ymin, GET_VALUE=ymin
  WIDGET_CONTROL, WID_TEXT_Ymax, GET_VALUE=ymax
  WIDGET_CONTROL, WID_DataType, GET_VALUE=value

  ; Data values to draw
  det_data = (value eq 0) ? tsd->get_value_tsd('FIS_OBS', 'DET'):tsd->get_value_tsd('FIS_OBS', 'FLUX')
  x = indgen(n_elements(det_data[0,*]), /L64)
  vdet_ch_num = n_elements(det_data[*,0])

  xrange = (n_elements(float(xmin)) NE 0) AND (n_elements(float(xmax)) NE 0) ? $
           [float(xmin), float(xmax)] : [0,0]
  yrange = (n_elements(float(ymin)) NE 0) AND (n_elements(float(ymax)) NE 0) ? $
           [float(ymin), float(ymax)] : [0,0]

  list_vdet_ps, det_data, xrange, yrange

  return

end

;-----------------------------------------------------------------
pro WID_BASE_LV_event, Event

  wTarget = (widget_info(Event.id,/NAME) eq 'TREE' ?  $
      widget_info(Event.id, /tree_root) : event.id)
  wWidget =  Event.top

  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='Close'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        CloseLV, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Left'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        ScrLeftLV, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Right'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        ScrRightLV, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Draw'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        DrawLV, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='PS'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        PsLV, Event
    end
    else:
  endcase

end

;-----------------------------------------------------------------
pro WID_BASE_LV, GROUP_LEADER=Group, _EXTRA=_VWBExtra_
COMMON Share
;COMMON Share_test, tsd ;For test
;tsd=Obj_New('tsd', INFILE='C:\RSI\makiuti\SikenData\Nagoya040412-\Nagoya040414\fis20040414_115011_03_dat_sw.fits')
;tsd=Obj_New('tsd', INFILE='C:\RSI\makiuti\SikenData\Nagoya040412-\Nagoya040414\fis20040414_125300_dat_lw.fits')

; Data values to draw ; initial data is DET
det_data=tsd->get_value_tsd('FIS_OBS', 'DET')
x = indgen(n_elements(det_data[0,*]), /L64)
vdet_ch_num = n_elements(det_data[*,0])
tsd_filename = tsd->get_filename()

;;;
WID_BASE_LV=widget_base(TITLE="VDET LIST: "+tsd_filename, GROUP_LEADER=group, /row)

;;;
cbase=widget_base(WID_BASE_LV, /frame, /column)
button=widget_button(cbase, value='Close', uname='Close', Tooltip='Close this window')

cbase1 = widget_base(cbase, /column)
t1 = widget_label(cbase1, value='Data')
datatype = [ 'DET', 'FLUX' ]
r1 = CW_BGROUP(cbase1, datatype, uvalue=datatype, uname='DataType', /EXCLUSIVE, /FRAME)
Widget_Control, r1, set_value=0; Select 'DET'

cbase2 = widget_base(cbase, /column)
t1 = widget_label(cbase2, value='Ymin')
t2 = widget_text(cbase2, value='', uname='Ymin', XSIZE=9, /EDITABLE)
t1 = widget_label(cbase2, value='Ymax')
t2 = widget_text(cbase2, value='', uname='Ymax', XSIZE=9, /EDITABLE)
t1 = widget_label(cbase2, value='Xmin')
t2 = widget_text(cbase2, value='', uname='Xmin', XSIZE=9, /EDITABLE)
t1 = widget_label(cbase2, value='Xmax')
t2 = widget_text(cbase2, value='', uname='Xmax', XSIZE=9, /EDITABLE)

sbase = widget_base(cbase2, /row)
button=widget_button(sbase, value='<<', uname='Left', Tooltip='Scroll to left')
button=widget_button(sbase, value='>>', uname='Right', Tooltip='Scroll to right')

button=widget_button(cbase, value='ReDraw', uname='Draw', YSize=30, Tooltip='ReDraw')

cbase3 = widget_base(cbase, /ROW, YPAD=10, /ALIGN_CENTER)
button=widget_button(cbase3, value='PS', uname='PS', XSize=70, Tooltip='Make a PS file')

;;;
draw=widget_draw(WID_BASE_LV, $
        xsize=Float(Getenv('Pix_Plot_X_SIZE'))*(vdet_ch_num/5), $
        ysize=Float(Getenv('Pix_Plot_Y_SIZE'))*5, $
        x_scroll_size=Float(Getenv('PList_X_SCR_SIZE')), $
        y_scroll_size=Float(Getenv('PList_Y_SCR_SIZE')), $
        uname='DrawWin')
widget_control, WID_BASE_LV, /realize


; Set the window for drawing
wset, !d.window
; Draw det_data list
!p.multi=[0,vdet_ch_num/5,5,0,0]
FOR i = 0, vdet_ch_num-1, 1 DO BEGIN

  ch_num=i+1

  IF (vdet_ch_num EQ 100) THEN $
    ch = 'SW Detector ch'+string(ch_num, format='(i3)') ;pixel to plot
  IF (vdet_ch_num EQ 75) THEN $
    ch = 'LW Detector ch'+string(ch_num, format='(i3)') ;pixel to plot
  ; Plot
  plot, x, det_data[i,*], TITLE=ch, charsize=1.4, /ynozero, $
      xstyle=3, xticks=2, XTITLE='Index', $
      background=getcolor('Black', !D.Table_Size-2), color=getcolor('Light Cyan', !D.Table_Size-3)

ENDFOR
!p.multi=0

;;;
XManager, 'WID_BASE_LV', WID_BASE_LV, /NO_BLOCK

end

;
; Empty stub procedure used for autoloading.
;
pro list_vdet, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
  WID_BASE_LV, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
end
