; $Id: make_initial_tree.pro,v 1.10 2006/12/11 06:33:25 makiuti Exp $
;+
; NAME:
;       MAKE_INITIAL_TREE
;
; PURPOSE:
;       This procedure makes the data selector tree widgets
;       as the FISv starts. Initial tree is for SW detector.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_initial_tree, widgetID
;
; MODIFICATION HISTORY:
;       Written by: Makiuti,S.
;       2006/12/11 Last modified
;-
pro make_initial_tree, widgetID

   ;Find the ID of the top-level base
   parentID = widgetID
   WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
      parentID = Widget_Info(parentID, /Parent)
   ENDWHILE

WID_BASE_8 = widget_info(parentID, find_by_uname='WID_BASE_8')
WID_BASE_9 = widget_info(parentID, find_by_uname='WID_BASE_9')
WID_TREE_0 = widget_info(parentID, find_by_uname='WID_TREE_0')
WID_TREE_1 = widget_info(parentID, find_by_uname='WID_TREE_1')


  ; ######## TREE_X ########

  ;  Make Tree
  ; (0) Index (Sample number)
  X_TREENODE_INDEX = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_INDEX'  $
      ,VALUE='Sample#', UVALUE='Sample#')

  ;  ## FIS_OBS Extension Root
  X_TREENODE_0 = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_0'  $
      ,/FOLDER ,/EXPANDED ,VALUE='FIS_OBS')

  ; (1) Time
  X_TREENODE_1 = Widget_Tree(X_TREENODE_0, UNAME='X_TREENODE_1'  $
      ,/FOLDER ,VALUE='Time')
    X_TREE_AFTIME = Widget_Tree(X_TREENODE_1, UNAME='X_TREE_AFTIME'  $
      ,VALUE='AFTIME',UVALUE='AFTIME')
    X_TREE_PIMTIORG = Widget_Tree(X_TREENODE_1, UNAME='X_TREE_PIMTIORG'  $
      ,VALUE='PIMTIORG',UVALUE='PIM_TI_ORG')
    X_TREE_FISTIORG = Widget_Tree(X_TREENODE_1, UNAME='X_TREE_FISTIORG'  $
      ,VALUE='FISTIORG', UVALUE='FIS_TI_ORG')
    X_TREE_PIMTI = Widget_Tree(X_TREENODE_1, UNAME='X_TREE_PIMTI'  $
      ,VALUE='PIMTI',UVALUE='PIM_TI_ITP')
    X_TREE_FISTI = Widget_Tree(X_TREENODE_1, UNAME='X_TREE_FISTI'  $
      ,VALUE='FISTI', UVALUE='FIS_TI_ITP')

  ; (2) Status
  X_TREENODE_2 = Widget_Tree(X_TREENODE_0, UNAME='X_TREENODE_2'  $
      ,/FOLDER ,VALUE='Status', UVALUE='XStatus')

  ; (3) Analog Telemetry
  X_TREENODE_3 = Widget_Tree(X_TREENODE_0, UNAME='X_TREENODE_3'  $
      ,/FOLDER ,VALUE='Analg Telemetry', UVALUE='XAnlgTele')

  ; (4) Detector Data
  X_TREENODE_4 = Widget_Tree(X_TREENODE_0, UNAME='X_TREENODE_4'  $
      ,/FOLDER, VALUE='Detector Data')

  X_TREENODE_48 = Widget_Tree(X_TREENODE_4, UNAME='X_TREENODE_48'  $
      ,/FOLDER, VALUE='DET', UVALUE='XVdet')
  X_TREENODE_49 = Widget_Tree(X_TREENODE_4, UNAME='X_TREENODE_49'  $
      ,/FOLDER ,VALUE='FLUX', UVALUE='XFlux')
  X_TREENODE_50 = Widget_Tree(X_TREENODE_4, UNAME='X_TREENODE_50'  $
      ,/FOLDER ,VALUE='FERR', UVALUE='XFerr')

  ; (5) Flags
  X_TREENODE_5 = Widget_Tree(X_TREENODE_0, UNAME='X_TREENODE_5'  $
      ,/FOLDER ,VALUE='Flags')

  X_TREE_BAD_FRAME = Widget_Tree(X_TREENODE_5, UNAME='X_TREE_BAD_FRAME'  $
      ,VALUE='BAD_FRAME', UVALUE='BAD_FRAME')
  X_TREE_UNDEF_ANOM_FRAME = Widget_Tree(X_TREENODE_5, UNAME='X_TREE_UNDEF_ANOM_FRAME'  $
      ,VALUE='UNDEF_ANOM_FRAME', UVALUE='UNDEF_ANOM_FRAME')
  X_TREE_BLANK = Widget_Tree(X_TREENODE_5, UNAME='X_TREE_BLANK'  $
      ,VALUE='BLANK', UVALUE='BLANK')
  X_TREE_IN_SAA = Widget_Tree(X_TREENODE_5, UNAME='X_TREE_IN_SAA'  $
      ,VALUE='IN_SAA', UVALUE='IN_SAA')
  X_TREE_NEAR_MOON = Widget_Tree(X_TREENODE_5, UNAME='X_TREE_NEAR_MOON'  $
      ,VALUE='NEAR_MOON', UVALUE='NEAR_MOON')
  X_TREE_UNTRUSTED_FRAME = Widget_Tree(X_TREENODE_5, UNAME='X_TREE_UNTRUSTED_FRAME'  $
      ,VALUE='UNTRUSTED_FRAME', UVALUE='UNTRUSTED_FRAME')

  X_TREE_PIX_FLAG = Widget_Tree(X_TREENODE_5, UNAME='X_TREE_PIX_FLAG'  $
    ,/FOLDER ,VALUE='Pixel_Flag', UVALUE='XFlag')

  ; (6) Quality
  X_TREE_QUALITY = Widget_Tree(X_TREENODE_0, UNAME='X_TREE_QUALITY'  $
      ,/FOLDER ,VALUE='Quality', UVALUE='XQual')

  ; (7) Counter
  X_TREE_COUNTER = Widget_Tree(X_TREENODE_0, UNAME='X_TREE_COUNTER'  $
      ,/FOLDER ,VALUE='Counter', UVALUE='XCunt')


  ;  ## FIS_HK Extension Root
  X_TREENODE_FIS_HK = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_FIS_HK'  $
      ,/FOLDER ,VALUE='FIS_HK', UVALUE='X_FIS_HK')

  ;  ## IRC_HK Extension Root
  X_TREENODE_IRC_HK = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_IRC_HK'  $
      ,/FOLDER ,VALUE='IRC_HK', UVALUE='X_IRC_HK')

  ;  ## HK_2 Extension Root
  X_TREENODE_HK_2 = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_HK_2'  $
      ,/FOLDER ,VALUE='HK_2', UVALUE='X_HK_2')

  ;  ## AOCU Extension Root
  X_TREENODE_AOCU = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_AOCU'  $
      ,/FOLDER ,VALUE='AOCU', UVALUE='X_AOCU')

  ;  ## ADS Extension Root
  X_TREENODE_GADS = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_GADS'  $
      ,/FOLDER ,VALUE='GADS', UVALUE='X_GADS')

  ;  ## PR Extension Root
  X_TREENODE_PR = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_PR'  $
      ,/FOLDER ,VALUE='PR', UVALUE='X_PR')

  ;  ## SE Extension Root
  X_TREENODE_SE = Widget_Tree(WID_TREE_0, UNAME='X_TREENODE_SE'  $
      ,/FOLDER ,VALUE='SE', UVALUE='X_SE')


  ; Select 'Index (Sample#)' by default
  Widget_control, X_TREENODE_INDEX, /SET_TREE_SELECT


  ; ######## TREE_Y ########

  ;  Make Tree
  ;  ## FIS_OBS Extension Root
  Y_TREENODE_0 = Widget_Tree(WID_TREE_1, UNAME='Y_TREENODE_0'  $
      ,/FOLDER ,/EXPANDED ,VALUE='FIS_OBS')

  ; (1) Time
  Y_TREENODE_1 = Widget_Tree(Y_TREENODE_0, UNAME='Y_TREENODE_1'  $
      ,/FOLDER ,VALUE='Time')
    Y_TREE_AFTIME = Widget_Tree(Y_TREENODE_1, UNAME='Y_TREE_AFTIME'  $
      ,VALUE='AFTIME',UVALUE='AFTIME')
    Y_TREE_PIMTIORG = Widget_Tree(Y_TREENODE_1, UNAME='Y_TREE_PIMTIORG'  $
      ,VALUE='PIMTIORG',UVALUE='PIM_TI_ORG')
    Y_TREE_FISTIORG = Widget_Tree(Y_TREENODE_1, UNAME='Y_TREE_FISTIORG'  $
      ,VALUE='FISTIORG', UVALUE='FIS_TI_ORG')
    Y_TREE_PIMTI = Widget_Tree(Y_TREENODE_1, UNAME='Y_TREE_PIMTI'  $
      ,VALUE='PIMTI',UVALUE='PIM_TI_ITP')
    Y_TREE_FISTI = Widget_Tree(Y_TREENODE_1, UNAME='Y_TREE_FISTI'  $
      ,VALUE='FISTI', UVALUE='FIS_TI_ITP')

  ; (2) Status
  Y_TREENODE_2 = Widget_Tree(Y_TREENODE_0, UNAME='Y_TREENODE_2'  $
      ,/FOLDER ,VALUE='Status', UVALUE='YStatus')

  ; (3) Analog Telemetry
  Y_TREENODE_3 = Widget_Tree(Y_TREENODE_0, UNAME='Y_TREENODE_3'  $
      ,/FOLDER ,VALUE='Analg Telemetry', UVALUE='YAnlgTele')

  ; (4) Detector data
  Y_TREENODE_4 = Widget_Tree(Y_TREENODE_0, UNAME='Y_TREENODE_4'  $
      ,/FOLDER, VALUE='Detector Data')

  Y_TREENODE_48 = Widget_Tree(Y_TREENODE_4, UNAME='Y_TREENODE_48'  $
      ,/FOLDER, VALUE='DET', UVALUE='YVdet')
  Y_TREENODE_49 = Widget_Tree(Y_TREENODE_4, UNAME='Y_TREENODE_49'  $
      ,/FOLDER ,VALUE='FLUX', UVALUE='YFlux')
  Y_TREENODE_50 = Widget_Tree(Y_TREENODE_4, UNAME='Y_TREENODE_50'  $
      ,/FOLDER ,VALUE='FERR', UVALUE='YFerr')

  ; (5) Flags
  Y_TREENODE_5 = Widget_Tree(Y_TREENODE_0, UNAME='Y_TREENODE_5'  $
      ,/FOLDER ,VALUE='Flags')

  Y_TREE_BAD_FRAME = Widget_Tree(Y_TREENODE_5, UNAME='Y_TREE_BAD_FRAME'  $
      ,VALUE='BAD_FRAME', UVALUE='BAD_FRAME')
  Y_TREE_UNDEF_ANOM_FRAME = Widget_Tree(Y_TREENODE_5, UNAME='Y_TREE_UNDEF_ANOM_FRAME'  $
      ,VALUE='UNDEF_ANOM_FRAME', UVALUE='UNDEF_ANOM_FRAME')
  Y_TREE_BLANK = Widget_Tree(Y_TREENODE_5, UNAME='Y_TREE_BLANK'  $
      ,VALUE='BLANK', UVALUE='BLANK')
  Y_TREE_IN_SAA = Widget_Tree(Y_TREENODE_5, UNAME='Y_TREE_IN_SAA'  $
      ,VALUE='IN_SAA', UVALUE='IN_SAA')
  Y_TREE_NEAR_MOON = Widget_Tree(Y_TREENODE_5, UNAME='Y_TREE_NEAR_MOON'  $
      ,VALUE='NEAR_MOON', UVALUE='NEAR_MOON')
  Y_TREE_UNTRUSTED_FRAME = Widget_Tree(Y_TREENODE_5, UNAME='Y_TREE_UNTRUSTED_FRAME'  $
      ,VALUE='UNTRUSTED_FRAME', UVALUE='UNTRUSTED_FRAME')

  Y_TREE_PIX_FLAG = Widget_Tree(Y_TREENODE_5, UNAME='Y_TREE_PIX_FLAG'  $
    ,/FOLDER ,VALUE='Pixel_Flag', UVALUE='YFlag')

  ; (6) Quality
  Y_TREE_QUALITY = Widget_Tree(Y_TREENODE_0, UNAME='Y_TREE_QUALITY'  $
      ,/FOLDER ,VALUE='Quality', UVALUE='YQual')

  ; (7) Counter
  Y_TREE_COUNTER = Widget_Tree(Y_TREENODE_0, UNAME='Y_TREE_COUNTER'  $
      ,/FOLDER ,VALUE='Counter', UVALUE='YCunt')


  ;  ## FIS_HK Extension Root
  Y_TREENODE_FIS_HK = Widget_Tree(WID_TREE_1, UNAME='Y_TREENODE_FIS_HK'  $
      ,/FOLDER ,VALUE='FIS_HK', UVALUE='Y_FIS_HK')

  ;  ## IRC_HK Extension Root
  Y_TREENODE_IRC_HK = Widget_Tree(WID_TREE_1, UNAME='Y_TREENODE_IRC_HK'  $
      ,/FOLDER ,VALUE='IRC_HK', UVALUE='Y_IRC_HK')

  ;  ## HK_2 Extension Root
  Y_TREENODE_HK_2 = Widget_Tree(WID_TREE_1, UNAME='Y_TREENODE_HK_2'  $
      ,/FOLDER ,VALUE='HK_2', UVALUE='Y_HK_2')

  ;  ## AOCU Extension Root
  Y_TREENODE_AOCU = Widget_Tree(WID_TREE_1, UNAME='Y_TREENODE_AOCU'  $
      ,/FOLDER ,VALUE='AOCU', UVALUE='Y_AOCU')

  ;  ## ADS Extension Root
  Y_TREENODE_GADS = Widget_Tree(WID_TREE_1, UNAME='Y_TREENODE_GADS'  $
      ,/FOLDER ,VALUE='GADS', UVALUE='Y_GADS')

  ;  ## PR Extension Root
  Y_TREENODE_PR = Widget_Tree(WID_TREE_1, UNAME='Y_TREENODE_PR'  $
      ,/FOLDER ,VALUE='PR', UVALUE='Y_PR')

  ;  ## SE Extension Root
  Y_TREENODE_SE = Widget_Tree(WID_TREE_1, UNAME='Y_TREENODE_SE'  $
      ,/FOLDER ,VALUE='SE', UVALUE='Y_SE')

end
