;+
; NAME:
;       MAKE_Y_QUALITY_CH_TREE
;
; PURPOSE:
;       This procedure makes the ch(pixel) number branches in each
;       quality node in the Y data selector of the FISv.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.ac.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_y_quality_ch_tree, widgetID, quality_type
;         'quality_type' is quality type (QUAL_RC_PARAM, QUAL_DF_EQ, etc.)
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;-
pro make_y_quality_ch_tree, widgetID, quality_type
Common Share

  parentID = widgetID
  WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
     parentID = Widget_Info(parentID, /Parent)
  ENDWHILE

  Widget_Control, parentID, UPDATE=0

  quality_type_member = ['QUAL_CV_PARAM', 'QUAL_RC_PARAM', 'QUAL_RC_CF', 'QUAL_DF_EQ', $
                         'QUAL_RP_DATA', 'QUAL_RP_PARAM', 'QUAL_RP_TABLE', 'QUAL_FF_PARAM', $
                         'QUAL_FF_CF', 'QUAL_GPGL_CORR', 'QUAL_MTGL_CORR', 'QUAL_TR_HIST', $
                         'QUAL_TR_PARAM', 'QUAL_DK_DATA', 'QUAL_DK_PARAM', 'QUAL_DK_TABLE', $
                         'QUAL_FX_CORR', 'QUAL_FX_PARAM']
  quality_type_header_member = ['QCVP', 'QRCP', 'QRCC', 'QDFE', $
                                'QRPD', 'QRPP', 'QRPT', 'QFFP', $
                                'QFFC', 'QGPC', 'QMTC', 'QTRH', $
                                'QTRP', 'QDKD', 'QDKP', 'QDKT', $
                                'QFXC', 'QFXP']
  quality_type_header = quality_type_header_member[where(quality_type EQ quality_type_member)]
  ; convert one-element array to scalar
  quality_type_header = quality_type_header[0]

  ; Parent branch ID in which child leaves are made
  temp_str1 = 'Y_QUALITY_'+quality_type_header
  Y_TREE_QUALITY_ID = widget_info(parentID, find_by_uname=temp_str1)

  ; Get detector type (SW or LW)
  pixmax = (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) ? 100:75

  FOR i = 1, pixmax DO BEGIN

    IF (i LE 9) THEN BEGIN
      pix_str = string(i, format='(I1)')
    ENDIF ELSE IF ((i GE 10) AND (i LE 99)) THEN BEGIN
      pix_str = string(i, format='(I2)')
    ENDIF ELSE IF (i EQ 100) THEN BEGIN
      pix_str = string(i, format='(I3)')
    ENDIF

      temp_str1 = quality_type_header + pix_str
      temp_id1  = Widget_Tree(Y_TREE_QUALITY_ID, UNAME='Y_QUALITY_'+pix_str $
        ,VALUE=pix_str, UVALUE=temp_str1)

  ENDFOR

  Widget_Control, parentID, /UPDATE

end