; $Id: maskflag.pro,v 1.4 2006/01/18 06:56:55 baba Exp $
;+
; NAME:
;       MASKFLAG
;
; PURPOSE:
;       The purpose of this procedure is to plot the data masked by flags.
;       Destination data are restricted to DET and FLUX.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.ac.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       maskflag, GROUP_LEADER=wGroup
;         wGroup is top level ID of the FISv
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        03/12/02 'BAD', 'DEAD', 'SATURATE', 'AFTERRST' flags can be used to mask.
;        05/12/09  Last modified
;-
;
;-----------------------------------------------------------------
; OK Button, do mask plot
;-----------------------------------------------------------------
pro maskflag_ok, Event
COMMON MASK_Share, fisv_top_id, mask_info
COMMON Share

; Get ID of DATA selector XTree
WID_TREE_0 = widget_info(fisv_top_id, find_by_uname='WID_TREE_0')
; Selected index in XTree
Selected_Xtree = widget_info(WID_TREE_0, /tree_select)
; Get uvalue of XTree
IF Selected_Xtree NE -1 THEN $
widget_control, Selected_Xtree, get_uvalue=xuvalue
; If no uvalue is available
IF (n_elements(xuvalue) EQ 0) THEN BEGIN
  result = DIALOG_MESSAGE(['X data must be selected.'], DIALOG_PARENT=fisv_top_id, /ERROR)
  print, 'X data must be selected.'
  WIDGET_CONTROL, Event.top, /DESTROY
  RETALL
ENDIF

; Get ID of DATA selector YTree
WID_TREE_1 = widget_info(fisv_top_id, find_by_uname='WID_TREE_1')
; Selected index in YTree
Selected_Ytree = widget_info(WID_TREE_1, /tree_select)
; Sort index of selected tree branch
Selected_Ytree = Selected_Ytree[sort(Selected_Ytree)]

; Discard data without valid uvalue
IF n_elements(Selected_Ytree) GT 1 THEN BEGIN
  valid_idx=-1
  FOR i=0, n_elements(Selected_Ytree)-1, 1 DO BEGIN
    widget_control, Selected_Ytree[i], get_uvalue=yuvalue
    IF (n_elements(yuvalue) NE 0) THEN valid_idx=[valid_idx, i]
  ENDFOR
  Selected_Ytree = Selected_Ytree[valid_idx[1:*]]
ENDIF

yuvalue_all=''
; Get uvalue of YTree
IF n_elements(Selected_Ytree) EQ 1 THEN BEGIN
    IF Selected_Ytree NE -1 THEN BEGIN
      widget_control, Selected_Ytree, get_uvalue=yuvalue

      ; If no yuvalue is available
      IF (n_elements(yuvalue) EQ 0) THEN BEGIN
        result = DIALOG_MESSAGE(['DET or FLUX must be selected.'], DIALOG_PARENT=fisv_top_id, /ERROR)
        print, 'DET or FLUX must be selected.'
        WIDGET_CONTROL, Event.top, /DESTROY
        RETALL
      ENDIF

      ; Set X&Y data values, titles
      xyData, xuvalue, yuvalue, x, y, xtitle, ytitle, title
      yuvalue_all=[yuvalue]

      ; If no X value is available
      IF (n_elements(x) EQ 0) THEN BEGIN
        result = DIALOG_MESSAGE(['X data must be selected.'], DIALOG_PARENT=fisv_top_id, /ERROR)
        print, 'X data must be selected.'
        WIDGET_CONTROL, Event.top, /DESTROY
        RETALL
      ENDIF
      ; If no Y value is available
      IF (n_elements(y) EQ 0) THEN BEGIN
        result = DIALOG_MESSAGE(['DET or FLUX must be selected.'], DIALOG_PARENT=fisv_top_id, /ERROR)
        print, 'DET or FLUX must be selected.'
        WIDGET_CONTROL, Event.top, /DESTROY
        RETALL
      ENDIF

      masked_x = x
      masked_y = y

    ; If no Y is selected
    ENDIF ELSE IF (Selected_Ytree EQ -1) THEN BEGIN
      result = DIALOG_MESSAGE(['DET or FLUX must be selected.'], DIALOG_PARENT=fisv_top_id, /ERROR)
      print, 'DET or FLUX must be selected.'
      WIDGET_CONTROL, Event.top, /DESTROY
      RETALL
    ENDIF

; If multiple Y are selected
ENDIF ELSE IF (n_elements(Selected_Ytree) GT 1) THEN BEGIN
    ; Show error message
    result = DIALOG_MESSAGE(["Too many data for mask plot!", "Just one pixel data of DET or FLUX is expected."],$
                DIALOG_PARENT=fisv_top_id, /ERROR)
    print, 'Too many data are selected!'

    ; Show error message in the message window
    WID_TEXT_0 = widget_info(fisv_top_id, find_by_uname='WID_TEXT_0')
    WIDGET_CONTROL, WID_TEXT_0, get_value = textvalue
    WIDGET_CONTROL, WID_TEXT_0, set_value = 'Too many data are selected!', $
      /append, SET_TEXT_TOP_LINE=n_elements(textvalue)-2

    WIDGET_CONTROL, Event.top, /DESTROY
    return
ENDIF

; If the data is VDET or FLUX, then get the pixel number. Eles, exit.
IF (strmatch(yuvalue, 'VDET*') or strmatch(yuvalue, 'FLUX*') ) EQ 1 THEN BEGIN
    ch = uint(strmid(yuvalue, 4,3))
ENDIF ELSE BEGIN
    result = DIALOG_MESSAGE(['Selected Y data must be DET or FLUX.'], DIALOG_PARENT=fisv_top_id, /ERROR)
    print, 'Selected Y data must be DET or FLUX.'
    WIDGET_CONTROL, Event.top, /DESTROY
    return
ENDELSE

Flag_Bad = widget_info(Event.top, find_by_uname='Flag_Bad')
flag_Bad_chk = Widget_info(Flag_BAD, /Button_Set)
Flag_Undef_Anom = widget_info(Event.top, find_by_uname='Flag_Undef_Anom')
flag_Undef_Anom_chk = Widget_info(Flag_Undef_Anom, /Button_Set)
Flag_Arith_Err = widget_info(Event.top, find_by_uname='Flag_Arith_Err')
flag_Arith_Err_chk = Widget_info(Flag_Arith_Err, /Button_Set)
Flag_Dead = widget_info(Event.top, find_by_uname='Flag_Dead')
flag_Dead_chk = Widget_info(Flag_Dead, /Button_Set)
Flag_Saturate = widget_info(Event.top, find_by_uname='Flag_Saturate')
flag_Saturate_chk = Widget_info(Flag_Saturate, /Button_Set)
Flag_Reset = widget_info(Event.top, find_by_uname='Flag_Reset')
flag_Reset_chk = Widget_info(Flag_Reset, /Button_Set)
Flag_Rstanom = widget_info(Event.top, find_by_uname='Flag_Rstanom')
flag_Rstanom_chk = Widget_info(Flag_Rstanom, /Button_Set)

; Information structure to keep the last parameters.
  mask_info = { $
    flag_Bad_chk:flag_Bad_chk,           $
    flag_Undef_Anom_chk:flag_Undef_Anom_chk, $
    flag_Arith_Err_chk:flag_Arith_Err_chk,   $
    flag_Dead_chk:flag_Dead_chk,         $
    flag_Saturate_chk:flag_Saturate_chk, $
    flag_Reset_chk:flag_Reset_chk,       $
    flag_Rstanom_chk:flag_Rstanom_chk }

; If any flags are checked
IF (flag_Bad_chk OR flag_Undef_Anom_chk OR flag_Arith_Err_chk OR flag_Dead_chk OR $
    flag_Saturate_chk OR flag_Reset_chk OR flag_Rstanom_chk) THEN BEGIN

    mask_index = indgen(n_elements(masked_y))

    ; Initialize
    Bad_flag=0
    Undef_Anom_flag=0
    Arith_Err_flag=0
    Dead_flag=0
    Saturate_flag=0
    Reset_flag=0
    Rstanom_flag=0
    ; Get flag values
    IF flag_Bad_chk THEN $
      Bad_flag=tsd->Get_Pix_Flag_TSD('FIS_OBS', 'Bad', PIXNO=ch-1)
    IF flag_Undef_Anom_chk THEN $
      Undef_Anom_flag=tsd->Get_Pix_Flag_TSD('FIS_OBS', 'Undef_Anom', PIXNO=ch-1)
    IF flag_Arith_Err_chk THEN $
      Arith_Err_flag=tsd->Get_Pix_Flag_TSD('FIS_OBS', 'Arith_Err', PIXNO=ch-1)
    IF flag_Dead_chk THEN $
      Dead_flag=tsd->Get_Pix_Flag_TSD('FIS_OBS', 'Dead', PIXNO=ch-1)
    IF flag_Saturate_chk THEN $
      Saturate_flag=tsd->Get_Pix_Flag_TSD('FIS_OBS', 'Saturate', PIXNO=ch-1)
    IF flag_Reset_chk THEN $
      Reset_flag=tsd->Get_Pix_Flag_TSD('FIS_OBS', 'Reset', PIXNO=ch-1)
    IF flag_Rstanom_chk THEN $
      Rstanom_flag=tsd->Get_Pix_Flag_TSD('FIS_OBS', 'Rstanom', PIXNO=ch-1)


    mask_index = where( (Bad_flag EQ 0) and (Undef_Anom_flag EQ 0) and (Arith_Err_Flag EQ 0) $
                        and (Dead_flag EQ 0) and (Saturate_flag EQ 0) and (Reset_flag EQ 0) and $
                        (Rstanom_flag EQ 0))

    ; If no data after masking
    IF (size(mask_index, /n_dimensions) EQ 0) THEN BEGIN
      print, 'There is no data to plot after masking.'
      Result = DIALOG_MESSAGE('There is no data to plot after masking.', /ERROR)
      WIDGET_CONTROL, Event.top, /DESTROY
      return
    ENDIF
    ;
    IF (n_elements(xuvalue) NE 0) AND (n_elements(y) NE 0) THEN BEGIN
      print, 'X:',xuvalue,', Y:',yuvalue_all

      tlb = widget_info(fisv_top_id, find_by_uname='FISV_BASE_0')
      ; Adjustment of xmargin & tickformat of plot in case of 'aftime'
      IF (strmatch(yuvalue, 'AFTIME')) then xmargin=[17.0, 3.0] else xmargin=[12.0, 3.0]
      IF (strmatch(xuvalue, 'AFTIME')) then xtickformat='(D13.3)' else xtickformat=''
      IF (strmatch(yuvalue, 'AFTIME')) then ytickformat='(D13.3)' else ytickformat=''
      ; Plot on another window
      zplot_alt, masked_x[mask_index], masked_y[mask_index]$
        , Y2DATA=y2, Y3DATA=y3, Y4DATA=y4, Y5DATA=y5 $
        , GROUP_LEADER=tlb $
        , ZOOM_XSIZE=Float(GetEnv('ZPlot_X_SIZE')), ZOOM_YSIZE=Float(GetEnv('ZPlot_Y_SIZE')) $
        , TITLE=title, XTITLE=xtitle, YTITLE=ytitle $
        , Y1TITLE=yuvalue, Y2TITLE=y2uvalue, Y3TITLE=y3uvalue $
        , Y4TITLE=y4uvalue, Y5TITLE=y5uvalue $
        , FILENAME = tsd->get_filename() $
        , CHARSIZE = (!d.name EQ 'X') ? 1.4 : 1 $
        , XMARGIN = xmargin, XTICKFORMAT=xtickformat, YTICKFORMAT=ytickformat $
        , EXTBTN_MODE=Fix(GetEnv('EXTBTN_MODE'))
    ENDIF
ENDIF ELSE BEGIN
    result = DIALOG_MESSAGE(['No flag is selected.'], DIALOG_PARENT=fisv_top_id, /ERROR)
    print, 'No flag is selected.'
ENDELSE

  WIDGET_CONTROL, Event.top, /DESTROY
  return

end
;-----------------------------------------------------------------
; Cancel Button
;-----------------------------------------------------------------
pro maskflag_cancel, Event

  WIDGET_CONTROL, Event.top, /DESTROY
  return

end
;-----------------------------------------------------------------
; Event check
;-----------------------------------------------------------------
pro MaskFlagBox_event, Event

  wTarget = (widget_info(Event.id,/NAME) eq 'TREE' ?  $
      widget_info(Event.id, /tree_root) : event.id)


  wWidget =  Event.top

  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='OK_BUTTON'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        maskflag_ok, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Cancel_BUTTON'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        maskflag_cancel, Event
    end
    else:
  endcase

end

pro MaskFlagBox, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
COMMON MASK_Share, fisv_top_id, mask_info

;  Resolve_Routine, 'maskflag_eventcb',/COMPILE_FULL_FILE  ; Load event callback routines

  ; Get geometry information of the FISv toplevel window
  fisv_top_geom = Widget_info(fisv_top_id, /GEOMETRY)

  MaskFlagBox = Widget_Base( GROUP_LEADER=wGroup, UNAME='MaskFlagBox' $
      , XOFFSET=fisv_top_geom.Xoffset+fisv_top_geom.Xsize $
      , YOFFSET=fisv_top_geom.Yoffset+fisv_top_geom.Ysize-300 $
      , TITLE='Mask CheckBox' , SPACE=3 , XPAD=3 , YPAD=3 $
      , COLUMN=1)

  ;
  MaskFlagBox_Base = Widget_Label(MaskFlagBox,  $
      UNAME='MaskFlagBox_Base' ,/ALIGN_CENTER  $
      ,VALUE='Flag to Mask')

  ;
  ButtonBase = Widget_Base(MaskFlagBox, UNAME='ButtonBase' ,FRAME=1  $
      ,XOFFSET=3 ,YOFFSET=20 ,/ALIGN_CENTER ,/BASE_ALIGN_CENTER  $
      ,TITLE='MaskPlot' ,SPACE=3 ,XPAD=3 ,YPAD=3 ,ROW=1)

  BASE_1 = Widget_Base(ButtonBase, UNAME='BASE_1' $
      ,TITLE='IDL' ,COLUMN=1 ,/NONEXCLUSIVE)

  Flag_Bad = Widget_Button(BASE_1, UNAME='Flag_Bad'  $
      ,/ALIGN_LEFT ,VALUE='Bad')

  Flag_Undef_Anom = Widget_Button(BASE_1, UNAME='Flag_Undef_Anom'  $
      ,/ALIGN_LEFT ,VALUE='Undef_Anom')

  Flag_Arith_Err = Widget_Button(BASE_1, UNAME='Flag_Arith_Err'  $
      ,/ALIGN_LEFT ,VALUE='Arith_Err')

  Flag_Dead = Widget_Button(BASE_1, UNAME='Flag_Dead'  $
      ,/ALIGN_LEFT ,VALUE='Dead')

  Flag_Saturate = Widget_Button(BASE_1, UNAME='Flag_Saturate'  $
      ,/ALIGN_LEFT ,VALUE='Saturate')

  Flag_Reset = Widget_Button(BASE_1, UNAME='Flag_Reset'  $
      ,/ALIGN_LEFT ,VALUE='Reset')

  Flag_Rstanom = Widget_Button(BASE_1, UNAME='Flag_Rstanom'  $
      ,/ALIGN_LEFT ,VALUE='RstAnom')

;  Flag_BUTTON_4 = Widget_Button(BASE_1, UNAME='Flag_BUTTON_4'  $
;      ,/ALIGN_LEFT ,VALUE='Button')
;
;  Flag_BUTTON_5 = Widget_Button(BASE_1, UNAME='Flag_BUTTON_5'  $
;      ,/ALIGN_LEFT ,VALUE='Button')


;  BASE_2 = Widget_Base(ButtonBase, UNAME='BASE_2' $
;      ,TITLE='IDL' ,COLUMN=1 ,/NONEXCLUSIVE)
;
;  Flag_BUTTON_6 = Widget_Button(BASE_2, UNAME='Flag_BUTTON_6'  $
;      ,/ALIGN_LEFT ,VALUE='Button')
;
;  Flag_BUTTON_7 = Widget_Button(BASE_2, UNAME='Flag_BUTTON_7'  $
;      ,/ALIGN_LEFT ,VALUE='Button')
;
;  Flag_BUTTON_8 = Widget_Button(BASE_2, UNAME='Flag_BUTTON_8'  $
;      ,/ALIGN_LEFT ,VALUE='Button')
;
;  Flag_BUTTON_9 = Widget_Button(BASE_2, UNAME='Flag_BUTTON_9'  $
;      ,/ALIGN_LEFT ,VALUE='Button')
;
;  Flag_BUTTON_10 = Widget_Button(BASE_2, UNAME='Flag_BUTTON_10'  $
;      ,/ALIGN_LEFT ,VALUE='Button')
;
;  Flag_BUTTON_11 = Widget_Button(BASE_2, UNAME='Flag_BUTTON_11'  $
;      ,/ALIGN_LEFT ,VALUE='Button')

  ;
  ButtonBase2 = Widget_Base(MaskFlagBox, UNAME='ButtonBase2'  $
      ,/ALIGN_CENTER ,ROW=1 ,/GRID_LAYOUT)


  OK_BUTTON = Widget_Button(ButtonBase2, UNAME='OK_BUTTON' $
      ,/ALIGN_CENTER ,VALUE='OK')


  Cancel_BUTTON = Widget_Button(ButtonBase2, UNAME='Cancel_BUTTON' $
      ,/ALIGN_CENTER ,VALUE='Cancel')

  IF n_elements(mask_info) THEN BEGIN
    IF mask_info.flag_Bad_chk        THEN Widget_Control, Flag_Bad, /SET_BUTTON
    IF mask_info.flag_Undef_Anom_chk THEN Widget_Control, Flag_Undef_Anom, /SET_BUTTON
    IF mask_info.flag_Arith_Err_chk  THEN Widget_Control, Flag_Arith_Err, /SET_BUTTON
    IF mask_info.flag_Dead_chk       THEN Widget_Control, Flag_Dead, /SET_BUTTON
    IF mask_info.flag_Saturate_chk   THEN Widget_Control, Flag_Saturate, /SET_BUTTON
    IF mask_info.flag_Reset_chk      THEN Widget_Control, Flag_Reset, /SET_BUTTON
    IF mask_info.flag_Rstanom_chk    THEN Widget_Control, Flag_Rstanom, /SET_BUTTON
  ENDIF


  Widget_Control, /REALIZE, MaskFlagBox


  XManager, 'MaskFlagBox', MaskFlagBox, /NO_BLOCK

end
;
;
pro maskflag, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
COMMON MASK_Share, fisv_top_id

  IF n_elements(wGroup) EQ 0 THEN wGroup=0
  fisv_top_id = wGroup

  MaskFlagBox, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
end
