;#> frin_plot_fringes.dc3
; Identifier	frin_plot_fringes
;
; Purpose	plot fringe components
;
; Synopsis	frin_plot_fringes, freq, ac, as, ch, cmin, optifreq, kk, $
;				   nfringes, frid
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		freq	 I     fltarr	fringe frequencies
;		ac	 I     fltarr	cosine amplitudes
;		as	 I     fltarr	sine amplitudes
;		ch	 I     fltarr	chi-sq values
;		cmin	 I     float	minimum value
;		optifreq I     float	best frequency
;		kk	 I     int	fringe number
;		nfringes I     int	total number allowed
;		frid	 I     struct	fringe identity
;
; Returns
;
; Description
;
;
;
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_plot_fringes.pro
;
; Author	Do Kester
;
; Version	1.2
;
; History	1.0  15-03-2001 DK, first implementation of frin_plot_fringes
;		1.1  05-04-2001 DK, remove compile_opt
;		1.2  16-05-2001 DK, remove minor tick marks on y-axis
;#<
;===========================================================================

pro frin_plot_fringes, freq, ac, as, ch, cmin, optifreq, kk, nfringes, frid

compile_opt idl2, hidden

common frinplotfringes, ycc, rc, oc

forward_function frin_title, empty_format

winset, 1, xsize=600, ysize=400, title='Fringe Amplitudes'

mplot = 4 < nfringes			; max # of plots in one screen
ysz = 0.8 / mplot			; ysize of plot window
yst = 0.1 + ( kk MOD mplot ) * ysz	; ystart of plotwindow
noerase = ( kk MOD mplot ) NE 0

pltitle =frin_title( frid )

pran = [ 0.1, yst, 0.9, yst + ysz ]
if ( kk EQ 0 ) then begin
  plot, freq, ac, pos=pran, xtitle="cycle / wavenumber", xst=3, ystyle=8, $
	yminor=1, yticklen=0.01

  ycc = cliprange( ch )
  rc = 0.9 * ( !y.crange[1] - !y.crange[0] ) / ( ycc[1] - ycc[0] )
  oc = !y.crange[0] - ycc[0] * rc
endif else begin
  ycr = !y.crange
  if ( ( kk MOD mplot ) EQ ( mplot - 1 ) ) then $
    plot, freq, ac, yr=ycr, pos=pran, xst=3, xtickformat='empty_format', $
	  noerase=noerase, title=pltitle, ystyle=8, yminor=1, yticklen=0.01 $
  else if ( ( kk MOD mplot ) EQ 0 ) then $
    plot, freq, ac, yr=ycr, pos=pran, xtitle="cycle / wavenumber", $
	  xst=3, ystyle=8, yminor=1, yticklen=0.01 $
  else plot, freq, ac, yr=ycr, pos=pran, xst=3, xtickformat='empty_format', $
	  noerase=noerase, ystyle=8, yminor=1, yticklen=0.01
endelse

oplot, freq, as, color=5

axis, yaxis=1, ystyle=19, yrange=ycc, color=2, yminor=1, yticklen=0.01
oplot, freq, ch * rc + oc, color = 2
dycc = ycc[1] - cmin
ycc = ycc - dycc
oc  = oc + dycc * rc

oplot, [optifreq, optifreq], [!y.crange[0],!y.crange[1]], color = 3
xyouts, 0.75, yst + 0.1 * ysz, /normal, $
	string( 'component ', kk+1, format='(a,i2)' )

return
end

