; $Id: extrac_data_by_rst.pro,v 1.3 2004/05/21 07:48:48 baba Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	extrac_data_by_rst.pro
; PURPOSE:
;	Data values between resets are extracted.
; CATEGORY:
;	Function. Libfary for the procedures in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = extrac_data_by_rst(tsd, rstcnt[, RSTTYPE=rsttype, COUNT=cnt) 
; INPUTS:
;	tsd		TSD object reference.
;	rstcnt		Integer. Reset count from which the data 
;			to be extracted
;	rsttype		Type of reset. Widel(0) or N170(1) for LW, 
;			and Wides(2) or N60(3) for SW.
;	pix		Integer type of pixel numbers to be 
;			negrected before and after the reset.
; OUTPUTS:
;	data 		Array of the detector data.
;	cnt		Number of resets within the TSD treated.
; RETURN:
;	data value	Value extracted
;	!FAIL in failure.
; RESTRICTIONS:
;	At this moment, only reset counter is checked.
;	Reset status should be checked in the future.
; PROCEDURE:
;		
; EXAMPLE:
;	ex_vdet = extrac_data_by_rst(tsd, 2)
; REFERENCE:
;	
;
; MODIFICATION HISTORY:
; 	Dec. 2002		Written by A. Kawamura.
;	Feb. 2003		reset counter is used instead of 
;				reset status
;-

FUNCTION extrac_data_by_rst, tsd, start_rstcnt, $
		RSTTYPE=rsttype, PIX=pix, CNT=cnt
;ON_ERROR 2

;;; Check for position parameters ;;;
	IF (N_PARAMS(0) LT 2) OR (N_PARAMS(0) GT 5)  THEN BEGIN
		PRINT, 'Calling Sequence: result=extrac_data_by_rst(tsd, start_rstcnt, rsttype[, pix=pix, cnt=cnt)'
		PRINT, 'TSD = Object reference type of TSD, read by WRITE_TSD or OBJ_NEW.'
		PRINT, 'RSTCNT = Reset count from which the data to be extracted'
		PRINT, 'RSTTYPE	= Type of reset. Widel=0, N170=1, Wides=2, N60=3.'
		PRINT, 'PIX = Pixel numbers to be negrected before or after the reset.'
		PRINT, 'CNT = Number of resets within the TSD treated.'
		PRINT, 'OUTPUT;	Array of the detector data.'
	RETURN,!FAIL
	ENDIF

;;; Extract values from TSD object reference ;;;
	
	vdet = tsd->get_value_tsd('fis_obs','flux')
	help, vdet
	si = SIZE(vdet)

;;; reset count ;;;
	CASE rsttype OF
		0:	rstcnt = tsd->get_value_tsd('fis_obs','rstcntwidel')
		1: 	rstcnt = tsd->get_value_tsd('fis_obs','rstcntwides')
		2:	rstcnt = tsd->get_value_tsd('fis_obs','rstcntn170')
		3:	rstcnt = tsd->get_value_tsd('fis_obs','rstcntn60')
		ELSE: 	IF si(1) EQ 100 then begin
			rstcnt = tsd->get_value_tsd('fis_obs','rstcntwides')
			ENDIF ELSE BEGIN
			rstcnt = tsd->get_value_tsd('fis_obs','rstcntwidel')
			ENDELSE
	ENDCASE

	diff_rstcnt = rstcnt - SHIFT(rstcnt, +1)

	ind_rstcnt = WHERE(diff_rstcnt NE 0, count)
	cnt = count
	print, cnt,' resets'

;	print,ind_rstcnt
	ind = WHERE(ind_rstcnt - SHIFT(ind_rstcnt, +1) EQ 0, count2)
	IF count2 NE 0 THEN $
	print,'reset count increases beyond 1 pix.'

;;; extracting the data ;;;;
	
	IF si(1) EQ 100 then begin
		start_ex = ind_rstcnt(start_rstcnt)+1
	ENDIF ELSE BEGIN
		start_ex = ind_rstcnt(start_rstcnt)
	ENDELSE

	ex_vdet=extrac(vdet,0,start_ex,$
		si[1],ind_rstcnt(1)-ind_rstcnt(0))

	RETURN, ex_vdet

END



