; $Id: extrac_data_by_time.pro,v 1.2 2004/05/21 07:48:48 baba Exp $
;
; Copyright (c) 2003, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	extrac_data_by_time.pro
; PURPOSE:
;	Data values specfied by CTIare extracted.
; CATEGORY:
;	Function. Libfary for the procedures in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = extrac_data_by_time(tsd, start[, end]) 
; INPUTS:
;	tsd		TSD object reference.
;	start		Integer. Starting time obtained by FIS TI counter.
;	end		Integer. End CTI(optional.) To the end of file
;			if not specified.	
; OUTPUTS:
;	data 		Array of the detector data.
;	cnt		Number of resets within the TSD treated.
; RETURN:
;	data value	Value extracted
;	!FAIL in failure.
; RESTRICTIONS:
;	At this moment, only CTI is to be specified
; PROCEDURE:
;		
; EXAMPLE:
;	ex_vdet = extrac_data_by_time(tsd, 2)
; REFERENCE:
;	
;
; MODIFICATION HISTORY:
; 	Mar. 2003		Written by A. Kawamura.
;-

FUNCTION extrac_data_by_time, tsd, startsample, endsample
;ON_ERROR 2

;;; Check for position parameters ;;;
	IF (N_PARAMS(0) EQ 0) THEN BEGIN
	PRINT, 'Calling Sequence: result=extrac_data_by_time(tsd, start[, end])'
	PRINT, 'TSD = Object reference type of TSD, read by WRITE_TSD or OBJ_NEW.'
	PRINT, 'start = sampling from which the data to be extracted'
	PRINT, 'end = End sampling until which the data to be extracted'
	PRINT, 'OUTPUT;	Array of the detector data.'
	RETURN,!FAIL
	ENDIF

;;; Extract values from TSD object reference ;;;
	
	vdet = tsd->get_value_tsd('fis_obs','det')
	help, vdet
	si = SIZE(vdet)

;;; Check TI ;;;
	startsample=startsample-1
	IF (startsample GT si(2)-1) THEN BEGIN
	PRINT, 'This data ends at',[si(2)-1]
	PRINT, 'NO Output...'
	RETURN,!FAIL
	ENDIF
	IF (startsample LT 0) THEN BEGIN
	PRINT, 'starting from the first sample'
	startsample=0
	ENDIF

	
	IF (N_PARAMS(0) EQ 2) THEN endsample =si(2)
	IF (endsample GT si(2)-1) OR (endsample LT 0) THEN BEGIN
		PRINT,'This data ends at',si(2),' samples'
		endsample =si(2)
	ENDIF

	endsample=endsample-1
	print,'from',startsample+1,' to',endsample+1,' samples'
	print, endsample-startsample+1,' samples'

;;; extracting the data ;;;;

	ex_vdet=vdet[*, startsample:endsample]

	RETURN, ex_vdet

END



