;
; Supplies function for cal_fit3_sw
;
; Define a return function for LMFIT:
;   f(x) = a[0]*exp(-t/a[1]) + a[2]*(1-exp(-t/a[3])) + a[4]*exp(-t/a[5]) + a[6]
Function cal3_sw_respons, t, a
  at = exp(-t/a[1])
  bt = exp(-t/a[3])
  dt = exp(-t/a[5])
  f  = a[0]*at + a[2]*(1.D -bt)+a[4]*dt+a[6]

  Return, [f, at, a[0]*at*t/a[1]^2, 1.D -bt, -a[2]*bt*t/a[3]^2, dt, a[4]*dt*t/a[5]^2, 1.D]
End

;
; Performs curve fitting with the following function and returns y(t1)
; where t1 may be an array
;
;  ref. S.H.Oh et al. 2004
;
Function gb_cal_fit3_sw, y, t, t1, YFIT=YFit, PARAMS=Params, TSTART=t0, CHISQ=chisq, FITA=FitA, ERR=err, ITMAX=itmax, ITMIN=itmin, ITER=iter

  ;  initial guess for the coefficients
  ;
  ndata = N_Elements(y)
  a = findgen(7)
  if keyword_set(Params) then begin
    a = Params
  endif else begin
    a[0] = 0.0
    a[2] = max(y[(ndata-1)/4:(ndata-1)*3/4])
    a[4] = 1.0*max(y[(ndata-1)/4:(ndata-1)*3/4])
    a[6] = y[0]-a[0]+a[4]*0.4
    a[0] = 10e-4 & a[2] = 1e-4 & a[4] = 5e-4 & a[6] = 1e-3

    a[1] = 200.
    a[3] = 1500.
    a[5] = 2000.
;     a[1] = 1100.
;     a[3] = 5000.
;     a[5] = 7000.
  endelse
  if NOT keyword_set(FitA) then fita = [1,1,1,0,1,0,1]
;  print, a
  IF ~Keyword_Set(t0) THEN BEGIN
    t0 = t[0]
  ENDIF
  t_0 = t - t0
  function_name = 'cal3_sw_respons'
  c = lmfit(t_0, y, a, FITA=fita,FUNCTION_NAME=function_name,CHISQ=chisq, $
            /double, CONVERGENCE=convergence, ITMAX=300, ITMIN=itmin, ITER=iter, MEASURE_ERROR=err)

  case convergence of
    1  : convergence=1 ; be quiet ;-)
    0  : LMESSAGE, 3, "% the algorithm '"+function_name+"' did not converge.",/CONTINUE
    -1 : LMESSAGE, 3, "% the algorithm '"+function_name+"' encountered a singular matrix and did not converge.",/CONTINUE
  endcase

  b = a[0]*exp(-(t1-t0)/a[1]) + a[2]*(1.D - exp(-(t1-t0)/a[3])) + a[4]*exp(-(t1-t0)/a[5]) + a[6]

  IF arg_present(YFit)   NE 0 THEN YFit = c   ; Fitted result value at each point
  IF arg_present(Params) NE 0 THEN Params = a ; Parameters of the fitting function

  Return, b

End
