; MODIFICATION HISTORY:
; 	Dec. 2003		Written by T.Takagi 
;       Nov. 2006		Modified by E.Figueredo
;       Jan. 2007               Modified by E.Figueredo
;

pro gb_iso_despiker, timeline, sigma, $
                                  threshold=threshold, $
                                  bad_freset = bad_freset, $
                                  replacement=replacement, $
                                  max_segment=max_segment,remove=remove,$ 
                                  glitches=glitches, debug = debug , $
                                  rise=rise, fall=fall,$
                                  subid=subid; for debug

if (n_elements(remove) eq 0) then remove = 0

data_orig = timeline 

if (n_elements(debug) eq 0) then debug = 0

if(n_elements(max_segment) gt 0) then begin
    n_elements_timeline = n_elements(timeline)
    n_segments = fix(n_elements_timeline/(1.0*max_segment))+1
    for i=0,n_segments-1 do begin
        imin = (i*max_segment - 1) > 0
        imax = ((i+1)*max_segment + 1) < (n_elements_timeline-1)
        if(imax-imin gt 3) then begin
            timeline_segment = timeline[imin:imax]
            bad_freset_segment = bad_freset[imin:imax]
            gb_iso_despiker, timeline_segment, $
              sigma, threshold=threshold, $
              bad_data = bad_data_segment, $
              replacement=replacement
            timeline[imin:imax] = timeline_segment
            bad_freset[imin:imax] = bad_freset_segment
        endif
    endfor 
endif 

if(n_elements(threshold) eq 0) then threshold = 4.0
if(n_elements(replacement) eq 0) then replacement='previous'

;;; 
;;; Calculate noise if not supplied
;;; 

if(n_elements(sigma) eq 0) then begin
    imax = n_elements(timeline) < 2000
    sample = timeline[0:imax-1]

;    diff_sample = median(sample[1:*] - sample[0:n_elements(sample)-2],3)
    diff_sample = sample[1:*] - sample[0:n_elements(sample)-2]

;    af_sky_stats, diff_sample, mu, var

;;; 
;;; Make a simple clipping to remove outliers
;;; 
    ordering = sort(abs(diff_sample-median(diff_sample)))    
    icut = fix(imax * 0.8)
    diff_sample = diff_sample[ordering[0:icut]]
;;; 
;;; Calculate noise level from this clipped data
;;; 
    ;; remove zero data
    id = where(diff_sample ne 0,cnt)
    if (cnt ne 0) then diff_sample = diff_sample[id]

    af_sky_stats, diff_sample, mu, var
    sigma = sqrt(var);*sqrt(5.0);/2.0
    lmessage, 7, 'Using noise level of '+strcompress(sigma), /cont
endif 


;;; 
;;; Identify glitches
;;; 

n_timeline = n_elements(timeline)

rise = timeline[1:*] - timeline[0:n_timeline-2]
fall = -1 * rise

rising_fast = rise gt threshold*sigma
falling_fast = fall gt threshold*sigma*0.66

glitches = where(rising_fast[0:n_timeline-2] and $
                 falling_fast[1:n_timeline-2], n_glitches) + 1

;;; Detecting Spikes around reset positions 

temp = where(bad_freset) -1
n_temp = n_elements(temp)
;print, temp, n_temp
for i=0,n_temp-1 do begin
    temp2=temp[i]
    if (temp2 gt 0) then begin
        if (timeline[temp2] gt threshold*sigma and $
            timeline[temp2-1] lt threshold*sigma) then begin
            glitches = append(glitches,temp2)
        endif
    endif
endfor

;;; Prevent bright sources has their peaks flagged out

diff = timeline - data_orig

n_glt = n_elements(glitches)

if (n_glt ge 1) and (glitches[0] gt 1) then begin
        new_gts = glitches(where(diff[glitches+1] eq 0) and where(diff[glitches-1] eq 0))
	glitches = new_gts
endif

;;; Double Spikes

glitches2 = where(rising_fast[0:n_timeline-3] and $
                 falling_fast[2:n_timeline-3], n_glitches) + 1

glitches = append(glitches,glitches2)
glitches3 = glitches2 + 1
glitches  = append(glitches,glitches3)

;;; 
;;; Remove glitches
;;; 

if (remove) then begin
    if(replacement eq 'previous' and n_glitches ne 0) then begin
        timeline[glitches] = timeline[glitches-2]

    endif 
    if(replacement eq 'linear' and n_glitches ne 0) then begin
        timeline[glitches] = 0.5*(timeline[glitches+2]+timeline[glitches-2])
    endif 
endif    

if (debug eq 1) then begin 
    isPlot = 1
    if (isPlot eq 1) then begin  
    !p.multi= [0,1,3]
    xr = [subid[0],subid[n_elements(subid)-1]]
    mu = median(timeline)
    sn = (timeline-mu)/sigma
    plot, subid,sn, yrange = [-10, 10],xr=xr,xstyle=1
    if (n_glitches ge 1) then begin
        mark=bytarr(n_elements(timeline))
        mark[*] = -100
        mark[glitches]=7
        oplot, subid, mark,psym=2
    endif

;    plot, subid,timeline, yrange = [-10*sigma +mu, 10*sigma+mu],xr=xr,xstyle=1
    plot, subid,rise/sigma, $
      yrange = [-10, 10],xr=xr,xstyle=1
    plot, subid,fall/sigma, $
      yrange = [-10, 10],xr=xr,xstyle=1

    wait,1
;    stop
    !p.multi = 0

    endif

endif
end
