; $Id: gb_set_fbad_at_nan.pro,v 1.2 2004/05/21 07:48:48 baba Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	gb_set_fbad_at_nan.pro
; PURPOSE:
;	Special values, such as NaN, Inf, are converted to float.
; CATEGORY:
;	Function. Library for ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = af_finite(infile, outfile, [value=value])
; INPUTS:
;	infile		array
;	value           value to be converted from special values.
;                       -9999. if not specified.
;			(optional.)
; OUTPUTS:
;	outfile         output file.
; RETURN:
;	fbad
; RESTRICTIONS:
;	
; PROCEDURE:
;
; EXAMPLE:
;       result = af_finite (vdet, finitevdet)
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	June  2003		Written by A. Kawamura.
;-

FUNCTION gb_set_fbad_at_nan, tsd, vdet=vdet
;ON_ERROR 2
IF N_PARAMS(0) EQ 0 THEN BEGIN
    MESSAGE,'FUNCTION af_finite, infile, outfile[,value=value]',/CONTINUE
    PRINT,'            infile = array'
    PRINT,'            value = value to be converted from special values.$'
    PRINT,'                    -9999. if not specified.(optional)'
    PRINT,'            outfile = output file.'

    RETURN, -1
ENDIF


;;; Reading input file ;;;
IF KEYWORD_SET(vdet) EQ 1 THEN BEGIN
	in_vdet=tsd->get_value_tsd('fis_obs','det')
ENDIF ELSE BEGIN
	in_vdet=tsd->get_value_tsd('fis_obs','flux')

        err=af_finite(in_vdet, vdet)
ENDELSE
  ;;; number of channels ;;;
	si = SIZE(vdet)
	num_ch = si[1]
	num_data = si[2]

fbad=tsd->get_pix_flag_tsd('fis_obs','bad')
HELP,fbad
;;; Search for NaN ;;;;
   MESSAGE,'Searching for NaN and INFINITE',/CONTINUE
   ind_nan = WHERE (FINITE(in_vdet, /NAN), count_nan)
   IF count_nan NE 0 THEN fbad[ind_nan] = 1
   ind_inf = WHERE (FINITE(in_vdet, /INFINITY), count_inf)
   IF count_inf NE 0 THEN fbad[ind_inf] = 1
   MESSAGE,'Flag, BAD, is set where values are NaN and/or INFINITY',/CONTINUE

    RETURN, fbad

END
