; $Id: gb_set_vl_with_flag.pro,v 1.2 2004/05/21 07:48:48 baba Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	gb_set_vl_with_flag.pro
; PURPOSE:
;
; CATEGORY:
;	Function. Green Box procedure in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = gb_set_vl_with_flag(tsd
;	[, CAL_CONST_FILE = cal_const_file, SUMMARY = summary])
; INPUTS:
;	tsd		TSD object reference.
;	cal_const_file	Calibration constants file; 
;			format, usage are TBD (optional.)
; OUTPUTS:
;
; RETURN:
;
; RESTRICTIONS:
;	
; PROCEDURE:
;	1. Extract detector data from TSD.
;	2. Read status.
;	3. Extract the data during the shutter close.
;	4. Examine the data extracted whether they satisfy 
;	the requirements for the dark data; the requirements are TBD.
;	5. Make the dark table, i.e. the list of the dark values along 
;	with the time. The format of the table is TBD.
;	6. Output the dark table.
;	7. Write summary. The contents of the summary with its 
;	management are TBD.
;
;	Requirements for the dark data are to be determined.
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	Oct. 2003		Written by A. Kawamura.
;-

FUNCTION gb_set_vl_with_flag, tsd, flux, flag_key, val=val
;ON_ERROR 2

;;; Starting GB_Make_Dark_Table ;;;
;   MESSAGE,'Executing GB_Make_Dark_Table...',/CONTINUE
	


  ;;; number of channels ;;;
	si = SIZE(flux)
	num_ch = si[1]
	num_data = si[2]

;;; Read status
   data_with_flag = tsd->get_pix_flag_tsd('fis_obs',flag_key)
   IND = WHERE(f_val NE 0, CNT)
   IF CNT NE 0 THEN flux[ind] = val

	return, flux
END
