;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	mmt_data_separator.pro
; PURPOSE:
;	Make segmented time series data for MMT
; CATEGORY:
;	Green Box procedure in library in ASTRO-F pipeline.
; CALLING SEQUENCE:
;
; INPUTS:
;	timeline        data array
;       n_data          Number of data in one segment
;       overlap         Overlap between two data segments
; OUTPUTS:
;       subArr          Segmented data 
;       subId           Data index of segmented data
;
; RESTRICTIONS:
;	
; PROCEDURE:
;	1. This function will be provided by the European team.
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	Mar. 2004		Written by T.Takagi
;    
;-

PRO mmt_data_separator, timeline, n_data=n_data, overlap=overlap, $
     subArr, subId, n_seg

;;;
;;; Check input parameters
;;;

;- invalid keywords
if (n_elements(n_data) eq 0) then begin
    MESSAGE,'Specify the data size of one segment!'
endif

if (n_elements(overlap) eq 0) then begin
    overlap = 0
endif

if (overlap ge n_data) then begin
    MESSAGE,'Invalid data overlap!'
endif


;- make sure n_data is integer
n_data = fix(n_data)


;;;
;;; Number of data segment
;;;
n_total = n_elements(timeline)
id = lindgen(n_total)
m = id mod (n_data-overlap)
zero_id = where(m eq 0,cnt) 
n_seg = cnt 
if ( m[n_total-1] le overlap ) then n_seg = n_seg-1

;;;
;;; Data segmentation
;;;

subArr = fltarr(n_seg, n_data)
subId = lonarr(n_seg, n_data)

;;; if n_seg=1, do mmt right now  
if (n_seg eq 1) then begin 
    subArr[0,*] = timeline
    subID[0,*] = id
    return
endif else begin 

    for i=0, n_seg-2 do begin 
        ibegin = zero_id[i]
        iend = ibegin + n_data-1
        subArr[i,*] = timeline[ibegin:iend]
        subID[i,*] = id[ibegin:iend]
    endfor

    ;- make last segment plus residual array
    subArr[n_seg-1,*] = timeline[n_total-n_data:n_total-1]
    subId[n_seg-1,*] = id[n_total-n_data:n_total-1]

endelse

return 
END
