;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	mmt_get_signal_width.pro
; PURPOSE:
;	Calculate width of signal at each side of a peak. The location
;	of peak should be given. 
; CATEGORY:
;	Green Box procedure in library in ASTRO-F pipeline.
; CALLING SEQUENCE:
;
; INPUTS:
;	data       input data 
;       peak       position of peak in data
;       frac       fraction of signal (1 at peak) at which widths 
;                  are mesured
;
; OUTPUTS:
;       width1     width of first half  (Note that width can be zero)
;       width2     width of second half 
;
;
; RESTRICTIONS:
;	
; PROCEDURE:
;
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	Aug. 2004		Written by T.Takagi
;    
;-

PRO mmt_get_signal_width, data, peak, width1, width2, frac=frac

if (n_elements(frac) eq 0) then frac=0.5

;; make sure that data is one dimensional array
dat = reform(data)

;; make positive signal 
if (dat[peak] lt 0) then begin
    dat = -dat
endif

maximum = dat[peak]

;; if there is no point to satisfy the condition below
width1 = peak
;;;- First half
for i=peak, 0, -1 do begin
    if (dat[i] le maximum*frac) then begin
        width1 = peak - i
        break
    endif
endfor

;; if there is no point to satisfy the condition below
width2 = n_elements(dat)-1 - peak
;;;- Second half
for i=peak, n_elements(dat)-1 do begin
    if (dat[i] le maximum*frac) then begin
        width2 = i - peak
        break
    endif
endfor

RETURN
END
