;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	mmt_subtract_cal_lamp
; PURPOSE:
;	subtract calibration lamp from data using the template
; CATEGORY:
;	Green Box procedure in library in ASTRO-F pipeline.
; CALLING SEQUENCE:
;
; INPUTS:
;       data            input timeline data
;       cal_lamp        a structure for cal-lamp templates
;       obs_mode        operation mode; i.e. Normal or CDS
; OUTPUTS:
;       output          output data
;
; RESTRICTIONS:
;	
; PROCEDURE:
;	1. This function will be provided by the European team.
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	Sep. 2006		Written by T.Takagi
;    
;-

PRO mmt_subtract_cal_lamp, num_ch, ch, data, cal_lamp, fcal, obs_mode, output

output = data

;;; number of channels ;;;
;si = SIZE(data)
;num_ch = si[1]
num_data = n_elements(data)

mask = mmt_isolate_signals(fcal, 0, sep_limit=10)
ncal = max(mask)

ntail = 120 ;; length of tail from Cal lamp


if (num_ch eq 75) then begin 
    template_norm = cal_lamp.norm_lw
    template_cds = cal_lamp.cds_lw
endif else begin 
    template_norm = cal_lamp.norm_sw
    template_cds = cal_lamp.cds_sw
endelse

for ncl=0,ncal-1 do begin 
    id = where(mask eq ncl+1)
    if ( n_elements(id) ge 10) then goto, longCal
    i1 = (id[n_elements(id)-1] - 20)        
    i3 = 0
    if (i1 lt 0) then begin 
        i3 = abs(i1)
        i1 = 0
    endif
    i2 = (i1+ntail-1-i3) < (num_data-1)
    ndat = (i2-i1)
    lamp = reform(data[i1:i2])
    isCDS = obs_mode[id[0]] eq 65 
    if (isCDS eq 1) then begin 
        template = reform(template_cds[ch,i3:*])
    endif else begin 
        template = reform(template_norm[ch,i3:*])
    endelse        
    
        ;;; Baseline estimation
    ii1 = (i1 - 80) > 0
    ii2 = i1+30 < (num_data-1)
    base1 = median(data[ii1:ii2])
    ii1 = i2
    ii2 = i2 + 30 < (num_data-1)
    base2 = median(data[ii1:ii2])
    if (i3 eq 0) then begin 
        base = (base1+base2)/2.
    endif else begin 
        base = base2
    endelse 
    lamp0 = lamp - base

        ;; Pick up around the peak
    iii1 = 15 < (n_elements(lamp0)-1)
    iii2 = 25 < (n_elements(lamp0)-1)
    mx1 = max(lamp0[iii1:iii2])
    mx2 = max(template[iii1:iii2])
    
    out1 = (lamp0 - template*mx1/mx2)+base
    output[i1:i2] = out1
    
    
;        device, decompose=0 
;        loadcolors
;        plot, lamp ,yrange=[0,(mx1+base)*1.3]
;        oplot, out1,color=5
;        oplot, fcal[i1:i2]*max(out1),psym=3
;        xyouts, 80, base*1.5, ch,charsize=3
;        xyouts, 80, base*1.3, ncl,charsize=3
;        wait, 0.3        

longCal:
endfor




return 
END
