Pro write_dk_fits, A, acomment, flnm

  ;; Create the primary header

  ; Find Time Stamp
  CALDAT, SYSTIME(/JULIAN, /UTC), mm, dd, yy, hh, ms, ss
  cdate = STRING(FORMAT='(I4.4,"-",I2.2,"-",I2.2,"T",I2.2,":",I2.2,":",I2.2)', $
            yy, mm, dd, hh, ms, ss, /PRINT)

  FXHMAKE,  header, /initialize, /extend, /date
  FXADDPAR, header, 'ORIGIN',   'ISAS/JAXA',    'Organization creating FITS file'
  FXADDPAR, header, 'DATE',     cdate,          'Date of data creation'
  FXADDPAR, header, 'DATATYPE', 'FIS DARK',     'Data Type'
  FXADDPAR, header, 'TELESCOP', 'ASTRO-F ',     'ASTRO-F mission'
  FXADDPAR, header, 'INSTRUME', 'FIS     ',     'FIS'
  FXADDPAR, header, 'TABLEVER', 2.0       ,     'Version of Dark Table'
  FXADDPAR, header, 'CREATOR',  'GB_MAKE_DARK_TABLE', 'Green Box module name and version'
  FXADDPAR, header, 'COMMENT',  '## FIS DARK TABLE ##', ''

  FXWRITE, flnm, header

  ;; Create the DARK extention header and write it

  sz_dark = size(A)

  FXBHMAKE, bhddark, sz_dark[1], 'DARK', 'Extension name'
  FXADDPAR, bhddark, 'EXTVER',  '1',                   'Extension version (must be an integer)'
  FXADDPAR, bhddark, 'TIMESYS', 'UTC',                 'Explicit time scale specification'
  FXADDPAR, bhddark, 'DATE-OBS','2006-02-21T21:28:00', 'Date of observation start'
  FXADDPAR, bhddark, 'DATE-END','2008-12-31T00:00:00', 'Date of observation end'
  FXADDPAR, bhddark, 'COMMENT', '', ''
  FXADDPAR, bhddark, 'COMMENT', 'Extracted from FIS observation packet', ''
  FXADDPAR, bhddark, 'COMMENT', acomment,  ''
  FXADDPAR, bhddark, 'COMMENT', '', ''

  FXBADDCOL, iAFTIME,  bhddark, 0D,  'AFTIME',  'ASTRO-F satellite time', TUNIT = 'sec'
  FXBADDCOL, iFLUX,    bhddark, 0E,  'FLUX',    'FIS dark flux data'
  FXBADDCOL, iERR,     bhddark, 0E,  'ERR',     'Error of dark flux'
  FXBADDCOL, iBAD,     bhddark, 0B,  'BAD',     'Bad On/Off'
  FXBADDCOL, iPID,     bhddark, 0B,  'PID',     'Packet ID'

  MWRFITS, A, flnm, bhddark, /NO_COMMENT

  return

END

; $Id: gb_make_dark_table.pro,v 1.25 2006/09/01 09:05:22 makiuti Exp $
;
; Copyright (c) 2002-2006, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
;
;FUNCTION NAME:
;        GB_Make_Dark_Table.pro
;
;PURPOSE:
;        The dark current level is extracted from the TSD data during the
;        shutter close and the cal. lamps off, and the dark values are
;        written to a dark table.  The dark table is updated by appending
;        new data as scan goes on.  This table may be used for a quick look
;        of dark level variation over time and for a GB module,
;        'gb_dark_subtraction'.
;
;CATEGORY:
;        Function. Green Box procedure in ASTRO-F pipeline.
;
;CALLING SEQUENCE:
;        Result = GB_Make_Dark_Table(tsd)
;
;INPUTS:
;        tsd ; TSD object reference.
;
;OUTPUT FILE FORMAT and NAMING:
;
;        The output file name will be DARK_LW_xxxx.fits or DARK_SW_xxxx.fits,
;        depending on the band.   The xxxx is associated with AFTIME when
;        data are taken.  Since AFTIME is expected to range ~2x10(8) sec.
;        [Aug.2005 - 2007], we divide them with 100000L, and use these
;        four digit values for xxxx.
;          ex)  aftime = 1.238938423828x10(8)  --> xxxx = 1238
;        Approximately 1 or 2 files per day will be generated. (too many??)
;
;        These files will be saved in directory ./DARKTABLE.  This directory
;        will be automatically generated, if it does not exist.
;
;        The output file is in fits extension form as follows.
;
;          1st column : aftime - DOUBLE AF Time
;          2nd column : flux   - FLOAT ARRAY with size of 100 (SW) or 75 (LW)
;          3rd column : ferr   - FLOAT ARRAY with size of 100 (SW) or 75 (LW)
;          4th column : bad    - BYTE ARRAY with size of 100 (SW) or 75 (LW)
;          5th column : pid    - BYTE Packet ID
;
;RETURN:
;        !SUCCEED if function ended successfully.
;        !FAIL in failure.
;
;PROCEDURE and DESCRIPTION:
;        1. Extract detector data and status mentioned above from TSD data
;           file.
;        2. Since in the beginning and last part, the cal. status bit,
;          CALPLSCNT(AS,AL,B) may be in error, we corrected them using
;          CAL(AS,AL,B)ON.  But this can not be perfect.  We just wanted
;          to reduce 'probability' of malfunctioning.
;        3. Extract the data with the followign status.
;
;             ( shutter is 0B ) AND ( calal(s)on is 0B ) AND
;             ( calbon is 0B ) AND ( l(s)wbooston is 0B ) AND
;             (in_saa is 0B) AND (blank is 0B)
;
;           whatelse??
;
;        4. We do not check other flags except for BAD flags which is
;           presumably collection of 'bad' status of each pixel.
;        5. Assign dark file name using AFTIME information.  There does not
;           have to be only one dark file.
;        6. Check if DARKTABLE directory exists, and if none, do 'mkdir.'
;        7. Check if the above named dark file exists, and if not, save dark
;           data to a new dark file.
;        8. If the dark file exist, read the dark file and sort data following
;           the sequence of AFTIMEs.  If there are overlaps in AFTIMEs, then
;           they are deleted.
;        9. A simple comment is also recorded in the new dark file.
;
;*****
;        In order to read/write dark table files which are not in TSD format,
;        several procedures in IDL ASTRONOMY LIBRARY havebeen used such as
;        MRDFITS, MWRFITS, FXHMAKE, FXADDPAR, FXWRITE, FXBHMAKE, and FXBADDCOL.
;
;
;REFERENCE:
;        Baba et al., ``ASTRO-F/FIS Survey Data Structure"
;
;MODIFICATION HISTORY:
;        Oct. 2002    Written by A. Kawamura.
;        May  2004    by YS: output file format is 'proposed'.
;        Aug. 2006    Last modified by Makiuti
;
;** CAUTIONS :
;(1) fldiv, idura, and idurb
;These are Variables one has to check and change accordingly, when dealing
;with REAL data : due to large size of TSD files, usually in test phase we
;often use rather simplified version of TSD, sometimes with a sparse time
;interval or with shorter cal. or dark measurement.
;
;-

FUNCTION GB_Make_Dark_Table, tsd, _extra=_extra
; ON_ERROR, 2

  ;;; Starting GB_Make_Dark_Table ;;;
  LMESSAGE, 5, 'Executing GB_Make_Dark_Table...', /CONTINUE

  ;;; Preliminary setting that need to be modified when dealing with REAL data
  fldiv=1.D5  ;  seconds, table file is generated at this rate

  ;;; Extract values from TSD object reference ;;;
  in_flux = tsd->Get_value_tsd('FIS_OBS','Flux')
  ret     = af_finite(in_flux, flux)
  verr    = tsd->Get_value_tsd('FIS_OBS','FErr')
  ti      = tsd->Get_value_tsd('FIS_OBS','AFtime')
  ; number of channels ;
    si = SIZE(flux)
    num_ch = si[1]
    num_data = si[2]

  ;;; Define duration of lighting by CalA for pulse(short) Cal.
  ;   ## This values need to be optimized. ##
  IF (num_ch EQ 75) THEN BEGIN ;; LW
    calAtime = 0.5     ; (05/11/26 since IT#4)
    idura = fix(calAtime*16.86)+1 ; Sampling rate is 16.86Hz in Survey mode
  ENDIF ELSE IF (num_ch EQ 100) THEN BEGIN ;; SW
    ;calAtime = 0.2     ; Old default
    ;calAtime = 0.1399  ; Crossing time of point source in survey obs.
    calAtime = 0.16    ; (05/11/26 since IT#4)
    idura = fix(calAtime*25.28)+1 ; Sampling rate is 25.28Hz in Survey mode
  ENDIF

  ;;; Read status and check if data are valid ;;;
  op_shutter = tsd->Get_status_tsd('FIS_OBS','ShtOp')  ; 1 for open shutter
  cl_shutter = 1 - op_shutter

  pid = tsd->Get_value_tsd('FIS_OBS','PacketID')

  IF (num_ch EQ 75) THEN BEGIN
    on_calA = tsd->Get_status_tsd('FIS_OBS','CalALon')
    calA_pls_cnt = tsd->Get_value_tsd('FIS_OBS','CalPlsCntAL')
    on_boost = tsd->Get_status_tsd('FIS_OBS','LWBoostOn')
    of_boost = 1 - on_boost
    flcom = '_LW_'
  ENDIF ELSE BEGIN
    on_calA = tsd->Get_status_tsd('FIS_OBS','CalASon')
    calA_pls_cnt = tsd->Get_value_tsd('FIS_OBS','CalPlsCntAS')
    on_boost = tsd->Get_status_tsd('FIS_OBS','SWBoostOn')
    of_boost = 1 - on_boost
    flcom = '_SW_'
  ENDELSE

  on_calB = tsd->Get_status_tsd('FIS_OBS','CalBon')
  of_calB = 1 - on_calB
  calB_pls_cnt = tsd->Get_value_tsd('FIS_OBS','CalPlsCntB')

  of_calA = 1 - on_calA


  ;;; to discard dark data taken at transition period to continuous (B) cal.,
  ;;; we superficially extend continuous (B) cal.
  ;
  idx_c=where(on_calB EQ 1, cnt)
  idx_b=where(idx_c - shift(idx_c, 1) NE  1)
  idx_e=where(idx_c - shift(idx_c,-1) NE -1)
  on_calB[idx_c[idx_b]-1]=1   &  on_calB[idx_c[idx_b]-2]=1
  on_calB[idx_c[idx_e]+1]=1   &  on_calB[idx_c[idx_e]+2]=1

  of_calB = 1 - on_calB

  st_bad   = tsd->get_pix_flag_tsd('FIS_OBS','Bad')
  no_bad   = 1 - st_bad

  ; probably we need more to check, e.g., bad pix and ....
  st_blank = tsd->get_flag_tsd('FIS_OBS','Blank')
  no_blank = 1 - st_blank
  in_in_saa= tsd->get_flag_tsd('FIS_OBS','In_SAA')
  no_in_saa= 1 - in_in_saa

;;-----------------------
;; Identify useful dark data frames with flags
;;-----------------------

  ;
  ; Avoid after effect of continuous CAL
  ;  - make usable flag which are set after continuous CAL
  calAstr = where(fix(on_calA) - shift(on_calA, 1) EQ  1, cnt_calAstr)
  calAend = where(fix(on_calA) - shift(on_calA, 1) EQ -1, cnt_calAend)

  IF (cnt_calAstr GT 0) AND (cnt_calAend GT 0) THEN BEGIN

    ; If file starts Cal On then void first calAend.
    IF (calAend[0] LT calAstr[0]) THEN calAend = calAend[1:N_elements(calAend)-1]
    ; If file ends Cal On then void last calAstr.
    IF (calAstr[N_elements(calAstr)-1] GT calAend[N_elements(calAend)-1]) THEN $
            calAstr = calAstr[0:N_elements(calAstr)-2]

    ; Frame# to be regarded as unusable for dark data after Cal ON
       ; Cal On more than 16 frames are regarded as continuous cal
       cont_cal_idx = where((calAend - calAstr) GT 16, cnt_unuse)
       IF (cnt_unuse GT 0) THEN BEGIN ; If there are continuous cal
         unuse_str = calAend[cont_cal_idx]
         ; 2000 frames after continuous cal are regarded as unusable for dark
         unuse_end = unuse_str + 2000

         unusable_dk = make_array(N_elements(on_calA), /BYTE)
         FOR i=0, N_elements(unuse_str)-1 DO BEGIN
           IF unuse_end[i] GE N_elements(unusable_dk) THEN unuse_end[i] = N_elements(unusable_dk)-1
           unusable_dk[unuse_str[i]:unuse_end[i]] = 1
         ENDFOR
       usable_dk = 1 - unusable_dk
       ENDIF

  ENDIF

  IF N_Elements(usable_dk) EQ 0 THEN usable_dk = make_array(N_elements(on_calA), value=1, /BYTE)

ok1_dk = cl_shutter * of_calA * usable_dk * of_calB * of_boost * no_in_saa ; '1' for dark frame
ok2_dk = ok1_dk * no_blank              ; '1' for dark & no_blank frame
ok3_dk = bytarr(num_ch, num_data)
FOR ch = 0, num_ch-1 DO BEGIN
  ok3_dk[ch,*]=ok2_dk[*] * no_bad[ch,*] ; '1' for dark & no_blank & no_bad frame
ENDFOR

;;-----------------------
;; Make index of dark data & Check them
;;-----------------------
idx_dk = where(ok2_dk NE 0.D, dk_cnt)
         ; idx_dk is an index of dark data frames
         ; dk_cnt is the number of dark data frames
idx_dkg = where(flux * ok3_dk NE 0.D, dk_cntg)
         ; idx_dkg is an index of dark data frames in good condition


IF (dk_cnt GT 5) THEN BEGIN ;; Now, needs more than 5 frames of dark data
  inum_str = where(idx_dk - shift(idx_dk, 1) NE  1, cnt_dkb) ; index# of dark start
  inum_end = where(idx_dk - shift(idx_dk,-1) NE -1         ) ; index# of dark end
                                                  ; cnt_dkb is a number of dark blocks

  ; Packet ID for each dark
  dk_packet_ID = pid[idx_dk[inum_str]]

  IF (cnt_dkb LE 2) THEN BEGIN
  ;; Originally, needs more than 2 sets of dark data in the file.
  ;; -> Now, allow single dark measurement.
    ;LMESSAGE, 3, 'Not sufficient dark data with reasonable intervals...Go back...', /CONTINUE
    ;return, !SUCCEED
    LMESSAGE, 5, 'Dark measurement is only once in the file.', /CONTINUE
  ENDIF
ENDIF ELSE BEGIN
  LMESSAGE, 3, 'No sufficient dark data in the file...Go back...', /CONTINUE
  return,!SUCCEED
ENDELSE

;;----------------------------------------------------------
;; Derive dark values
;;----------------------------------------------------------
dk_flux = dblarr(num_ch, cnt_dkb) ; cnt_dkb is the number of dark data set (block) in the file
dk_err  = dblarr(num_ch, cnt_dkb)
dk_bad  = intarr(num_ch, cnt_dkb) ; -> if dark data is bad, =1

;;-----------------------
;; Loop for channel (pixel)
;;-----------------------
FOR ch = 0, num_ch-1 DO BEGIN

  ;;-----------------------
  ;; Estimate a dark value at each dark data set.
  ;;-----------------------
  dk_time = dblarr(cnt_dkb) ; cnt_dkb is the number of dark data set (block) in the file

  FOR j = 0, cnt_dkb-1 DO BEGIN

    ;; not to use data at transition period
    IF (num_ch EQ 75) AND (ch EQ 0) THEN BEGIN ; If pixel with strong after effect
                                               ;;;  now special pixel is LW#1
      nouse_aft_pcal = 160
    ENDIF ELSE BEGIN
      nouse_aft_pcal = 10
    ENDELSE
    ii_str = idx_dk[inum_str[j]] + nouse_aft_pcal ; start frame# of 'j'th dark data
    ii_end = idx_dk[inum_end[j]] - 2              ; end   frame#

    IF (ii_end LE ii_str) THEN Continue ; avoid an error due to invalid dark range

    ;idx_ok = where(flux[ch,ii_str:ii_end]*ok3_dk[ch, ii_str:ii_end] ne 0.D, cntp)
    idx_ok   = where(ok3_dk[ch, ii_str:ii_end] GT 0.D, cntp)
    idx_ok_f = where(ok2_dk[ii_str:ii_end] GT 0.D) ; This index isn't affected by pixel bad flag.

    ; check if whole data in a block are bad
    IF (cntp LE 5) THEN BEGIN
      msg = 'Not enough Dark data in the'+strcompress(string(j+1))$
            +'th dark of ch.'+strcompress(string(ch))
      dk_bad[ch, j] = 1
      LMessage, 7, msg, /Continue
      Continue
    ENDIF

    ;; Set dark value for 'j'th dark data
    dk_flux[ch, j] = gb_dark_fit(flux[ch, idx_ok + ii_str], ti[idx_ok], ERR=err, CHISQ=chisq)
                                                           ;ti[idx_ok+ii_str] is also acceptable
    dk_err[ch, j]  = err
    dk_time[j] = mean(ti[idx_ok_f + ii_str])
    IF (err EQ 0) THEN dk_bad[ch, j] = 1

    ; To set quality (tentative code at 050609, the definition is groundless)
    ;print, ch, ii_str, chisq; #for test
;    IF ((chisq GT 2e-7) AND (chisq LE 5e-7)) THEN BEGIN
;      ;print, 'QUAL1', ch, ii_str, chisq; #for test
;      IF (j GE 1) THEN BEGIN
;        qual_dk_data[ch, idx_dk[inum_str[j-1]]+2:ii_str]=1
;        qual_dk_table[ch, idx_dk[inum_str[j-1]]+2:ii_str]=1
;      ENDIF ELSE IF (j EQ 0) THEN BEGIN
;        qual_dk_data[ch, 0:ii_str]=1
;        qual_dk_table[ch, 0:ii_str]=1
;      ENDIF
;    ENDIF ELSE IF (chisq GT 5e-7) THEN BEGIN
;      ;print, 'QUAL2', ch, ii_str, chisq; #for test
;      IF (j GE 1) THEN BEGIN
;        qual_dk_data[ch, idx_dk[inum_str[j-1]]+2:ii_str]=2
;        qual_dk_table[ch, idx_dk[inum_str[j-1]]+2:ii_str]=2
;      ENDIF ELSE IF (j EQ 0) THEN BEGIN
;        qual_dk_data[ch, 0:ii_str]=2
;        qual_dk_table[ch, 0:ii_str]=2
;      ENDIF
;    ENDIF

  ENDFOR

ENDFOR ;; End of channel loop

; Remove invalid data
idx_valid = where(dk_time NE 0.0, cnt_valid)
IF (cnt_valid GT 0) THEN BEGIN
  dk_flux      = dk_flux[*, idx_valid]
  dk_err       = dk_err[*, idx_valid]
  dk_time      = dk_time[idx_valid]
  dk_packet_ID = dk_packet_ID[idx_valid]
ENDIF ELSE BEGIN
  msg = '    The dark table remains the same.'
  LMessage, 5, msg, /PRINT
  Return, !SUCCEED
ENDELSE

;;----------------------------------------------------------
;;  Write into dark table files (FITS format)
;;----------------------------------------------------------
  ; Define structure
  _A = {aftime:0D, flux:fltarr(num_ch), err:fltarr(num_ch), bad:bytarr(num_ch), pid:0B}

  ; Make subdirectory to save table files
   ret = file_test('DARKTABLE', /dir)
   IF (ret EQ 0) THEN BEGIN
     file_mkdir, 'DARKTABLE'
     LMessage, 5,' DARKTABLE directory generated.', /Continue
   ENDIF

  ; Assortment (Find ID numbers for dark files)
  tidn = fix(dk_time[*]/fldiv) ; dk_time is in AF-time; fldiv is an interval of the file.
                               ; tidn is the ID number of dark files to be generated/updated
  idx_idc = where(tidn - shift(tidn, 1) NE 0, cntds) ; index# of changing points of tidn
                                            ; cntds is the number of changing points of tidn

  ; Check if ID number of table file is valid.
  IF (total(tidn) EQ 0) THEN BEGIN ; If tidn[*] EQ 0
    LMessage, 5, 'DarkTable file name is NOT VALID.', /Continue
    Return, !FAIL
  ENDIF

  IF (cntds EQ 0) THEN BEGIN ; If all data is to be in one file,
    cntds = 1         ; only one file will be generated/updated
    idx_idc = [0, N_elements(tidn)]
  ENDIF ELSE BEGIN
    idx_idc = [idx_idc, N_elements(tidn)]
  ENDELSE

  ; cntds is the number of dark files to be generated/updated
  FOR i = 0, cntds-1 DO BEGIN

    fileID = tidn[idx_idc[i]]
    iis = idx_idc[i]
    iie = idx_idc[i+1]-1

    ;; Make contents of FITS table
    num = iie - iis + 1
    A   = replicate(_A, num)
    ;
    A.aftime = dk_time[iis:iie]
    A.flux   = dk_flux[*, iis:iie]
    A.err    = dk_err[*, iis:iie]
    A.bad    = dk_bad[*, iis:iie]
    A.pid    = dk_packet_ID[iis:iie]

     ; See whether dark file already exists
     flnm = './DARKTABLE/DARK'+flcom+string(fileID, format='(i4.4)')+'.fits'
     dum = file_search(flnm)

     IF (dum EQ '') THEN BEGIN    ; the file does not exist

       Acomment = '    A new dark table file generated.'
       write_dk_fits, A, acomment, flnm

       LMessage, 5, acomment+flnm, /PRINT

     ENDIF ELSE BEGIN    ; the file does exist and thus should be updated

       DK = MRDFITS(flnm, 1, STATUS=status)
       dsz = size(DK)
       A = [DK, A]
       A = A[sort(A.aftime)]
       idx = where(A.aftime - shift(A.aftime, 1) NE 0.D, cnt)
       IF (cnt GT 0) THEN A = A[idx]
       asz = size(A)
       Acomment = '    The dark table is updated.'
       IF (dsz[1] EQ asz[1]) THEN Acomment = '    The dark table remains the same.'
       write_dk_fits, A, acomment, flnm

       LMessage, 5, acomment+flnm, /PRINT

     ENDELSE

  ENDFOR

  Return, !SUCCEED

END
