; $Id: detector_offset_ccf.pro,v 1.4 2006/05/12 07:34:03 do Exp $
;
; Copyright (c) 2003, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	detector_offset_ccf.pro
; PURPOSE:
;	obtain the detector offsets in arcmin wrt boresight
; CATEGORY:
;	CCF-Function. Library for the procedures in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	offset = detector_offset_ccf( ccf, tsd, det, band=band, $
;	    nominal=nominal, error=error )
; INPUTS:
;	ccf		calibration constants file
;			if present read the file (anew)
;	tsd		tsd, if present check the time range
;	det		detector number (starting at 1)
;			default: first one
;			when det = 0, it returns [0,0] or boresight
;	band		either All, Wide or Narrow. One letter suffices
;			default: All
;	nominal 	keyword. if set give nominal (non-distorted) position
;			default: distorted
;	error		error status (!SUCCEED or !FAIL)
;
; OUTPUTS:
; RETURN:
;	offset		2 offsets
;			offset[0]  x-position
;			offset[1]  y-position
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
;	June 2005		Written by Do Kester.
;-

FUNCTION detector_offset_ccf, ccf, tsd, det=detin, band=band, $
	    nominal=nominal, error=error

COMPILE_OPT IDL2

COMMON detoffset_ccf, ccf_detoff

if ( n_elements( detin ) EQ 0 ) then det = 1 else det = detin
if ( det EQ 0 ) then return, [0.0,0.0]		    ; boresight

if ( n_params( ) GT 0 ) then $
    ccf_detoff = ccf_read_ccf( ccf, tsd, error=error )

nd = ccf_detoff.ndet

if ( keyword_set( nominal ) ) then begin
    x = ccf_detoff.x_nominal
    y = ccf_detoff.y_nominal
endif else begin
    x = ccf_detoff.x_distorted
    y = ccf_detoff.y_distorted
endelse

;plot, ccf_detoff.x_nominal, ccf_detoff.y_nominal, ps=1, xr=[0,20],yr=[5,25]
;q = indgen( 15 )
;for k = 0,4 do $
;oplot, (ccf_detoff.x_distorted)[q+15*k], (ccf_detoff.y_distorted)[q+15*k], $
;ps=1, color=2+k


if ( n_elements( band ) GT 0 ) then begin
    b = strupcase( strmid( band, 0, 1 ) )
    if ( b EQ 'N' ) then det += ( nd * 3 ) / 5
endif


return, [x[det-1], y[det-1]]
end

