; $Id: im_mtfirstflag.pro,v 1.2 2006/01/18 06:57:10 baba Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	im_mtfirstflag.pro
; PURPOSE:
;	get the starting sample of a series of mt glitch flag
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	flag = im_mtfirstflag( tsd )
;
; INPUTS:
;	tsd		a tsd object
;
; RETURN
;	flag		first flag of each series of mt flags (per type)
;
; PROCEDURE:
;	For each type the glmt_type* flag is read.
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;	Nov. 2005		Written by Do Kester
;-



function im_mtfirstflag, tsd

for k = 1, 4 do begin
    type = string( 'mtgl_type', k, format='(a,i1)' )
    flag = tsd->get_pix_flag_tsd( 'fis_obs', type )
    flag = flag GT shift( flag, 0, 1 )
    if ( k EQ 1 ) then glfl = flag else glfl = glfl OR flag
endfor


return, glfl
end
