;#> lu_logdet.dc3
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Function:  lu_logdet
;
; Purpose:   Calculation of determinant
;
; Header:    ldet = lu_logdet, mat, parity = parity
;		mat	real/double	IN : LU decomposed matrix from lu_decomp
;		parity  integer		IN : parity as from lu_decomp
;					OUT: sign of the determinant
;
; Returns:   ldet	real/double	OUT: log10( abs( det of the mat ) )
;
; Notes:     adapted from Numerical Recipes, more or less
;	      The determinant of the inverse of the matrix = 1 / det	      
;
; History:   DK	24/10/95
;-----------------------------------------------------------------------------
;#<

function lu_logdet, mat, parity=parity

sz = size( mat )
b = indgen( sz(1) )
diag = mat(b,b)
b = where( diag EQ 0, nb )
if ( nb GT 0 ) then begin
  parity = 0
  return, 0
endif
b = where( diag LT 0, nb )
for k = 1, nb do parity = - parity

return, total( alog10( abs( diag ) ) )

end
