;#> lu_stdev.dc3
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Function:  lu_stdev
;
; Purpose:   Calculation of standard deviations pertaining to the solution
;
; Header:    stdev = lu_stdev( mat, index, chisq, covar=covar )
;		mat	real/double	IN : LU decomposed matrix from lu_decomp
;		index	integer 	IN : permutation index from lu_decomp
;		chisq	real/double	IN : as calculated with lu_chisq
;		covar	real/double	OUT: covariance matrix
;
; Returns:   stdev	real/double	OUT: array of standard deviations
;
; Notes:     adapted from Numerical Recipes, more or less
;	      In those cases where the variance turns out negative,
;	      the values for the stdev is returned as
;		stdev = - sqrt( fabs( variance ) )
;	      This could be an indication of degeneracy between parameters.
;
; History:   DK 24/10/95
;	     17/08/99 DK, add covar option && make sure it is symmetric
;-----------------------------------------------------------------------------
;#<

function lu_stdev, mat, index, chisq, covar=covar

forward_function lu_inverse

st = reform( mat[0,*] )
nr = n_elements( st )

covar = chisq * lu_inverse( mat, index )

for j = 0, nr-1 do begin
  st[j] = covar[j,j]
  if ( st[j] GE 0 ) then st[j] = sqrt( st[j] ) $
  else			 st[j] = -sqrt( abs( st[j] ) )
  covar[j,*] = 0.5 * ( covar[j,*] + covar[*,j] )	; make covar symmetric
  covar[*,j] = covar[j,*]
endfor

return, st
end
