; $Id: fis_obs__define.pro,v 1.4 2006/10/10 09:28:48 cyamauch Exp $
;
; Copyright(C) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; CLASS_NAME:
;   FIS_OBS - Binary Table Extension of FITS, inherited from BTE
;
; PURPOSE:
;   Define the structure for HDU, which is a part of TSD.
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   None
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   None
;
; DESCRIPTION:
;   See 'ASTRO-F FIS/Survey Data Structure' in detail.
;
; EXAMPLE:
;   None
;
; SIDE EFFECTS:
;   None
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, June 2002.
;
;-

;----------------------------------------------------------------------------
; Below is the definitions of FIS_OBS specific functions.
;----------------------------------------------------------------------------

; FUNCTION fis_obs::Get_Status_Members
;     COMPILE_OPT hidden
;     IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;     CASE self.extver OF
;         4: members = $ ; number = 21
;           [ 'CREON', 'SHTOP', 'FWPOSON', 'FWPOS_B1', $
;             'FWPOS_B0', 'MPOSON', 'MPOS_B1', 'MPOS_B0', $
;             'RSTWIDELON', 'RSTWIDESON', 'RSTN170ON', 'RSTN60ON', $
;             'SWBOOSTON', 'LWBOOSTON', 'SWBIASON', 'LWBIASON', $
;             'CALALON', 'CALASON', 'CALBON', 'SINALON', 'SINASON' ]
;         3: members = $ ; number = 21
;           [ '', '', '', '', '', '', '', '', '', '', '', 'SINASON', $
;             'SINALON', 'CALBON', 'CALASON', 'CALALON', $
;             'LWBIASON', 'SWBIASON', 'LWBOOSTON', 'SWBOOSTON', $
;             'RSTN60ON', 'RSTN170ON', 'RSTWIDESON', 'RSTWIDELON', $
;             'MPOSB0', 'MPOSB1', 'MPOSON', 'FWPOSB0', $
;             'FWPOSB1', 'FWPOSON', 'SHTOP', 'CREON' ]
;         2: members = $ ; number = 21
;           [ '', '', '', '', '', '', '', '', '', '', '', 'SINASON', $
;             'SINALON', 'CALBON', 'CALASON', 'CALALON', $
;             'LWBIASON', 'SWBIASON', 'LWBOOSTON', 'SWBOOSTON', $
;             'RSTN60ON', 'RSTN170ON', 'RSTWIDESON', 'RSTWIDELON', $
;             'MPOSB0', 'MPOSB1', 'MPOSON', 'FWPOSB0', $
;             'FWPOSB1', 'FWPOSON', 'SHTOP', 'CREON' ]
;         1: members = $ ; number = 20
;           [ '', '', '', '', '', '', '', '', '', '', '', '', $
;             'SINASON', 'SINALON', 'CALBON', 'CALASON', $
;             'CALALON', 'LWBIASON', 'SWBIASON', 'LWBOOSTON', $
;             'SWBOOSTON', 'RSTN60ON', 'RSTN170ON', 'RSTWIDESON', $
;             'RSTWIDELON', 'MPOSON', 'FWPOSON', 'SHTOP', $
;             'CREON', 'OBSMODE', 'DETECTOR', 'BLANK' ]
;         ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
;     ENDCASE
;     RETURN, members
; END

; ; aka. frame_flag
; FUNCTION fis_obs::Get_Flag_Members
;     COMPILE_OPT hidden
;     IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;     CASE self.extver OF
;         4: members = $
;           [ 'BAD_FRAME', 'UNDEF_ANOM_FRAME', 'BLANK', 'IN_SAA', 'NEAR_MOON', 'UNTRUSTED_FRAME' ]
;         3: members = [ '', '', '', 'NEAR_MOON', $
;                        'IN_SAA', 'BLANK', 'UNDEF_ANOM_FRAME', 'BAD_FRAME' ]
;         2: members = [ '', '', '', 'BADRCDW', 'BADRCDN', 'NEAR_MOON', 'IN_SAA', 'BLANK' ]
;         1: members = [ '', '', '', 'BADRCDW', 'BADRCDN', 'NEAR_MOON', 'IN_SAA', 'BLANK' ]
;         ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
;     ENDCASE
;     RETURN, members
; END

; FUNCTION fis_obs::Get_Pix_Flag_Members
;     COMPILE_OPT hidden
;     IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;     CASE self.extver OF
;         4: members = $ ; number = 29/32
;           [ 'BAD', 'UNDEF_ANOM', 'ARITH_ERR', 'DEAD', $
;             'SATURATE', 'RESET', 'RSTANOM', 'NO_DIFF', $
;             'NO_RP_CORR', 'NO_DK_CORR', 'NO_DCCAL_CORR', 'NO_TR_CORR', $
;             'NO_FLAT_FIELD', 'NO_GPGL', 'NO_MTGL', 'NO_ABSCAL', $
;             'GPGL_TYPE1', 'GPGL_TYPE2', 'GPGL_TYPE3', 'GPGL_TYPE4', $
;             'GPGL_TAIL', $
;             'MTGL_TYPE1', 'MTGL_TYPE2', 'MTGL_TYPE3', 'MTGL_TYPE4', $
;             'MTGL_TAIL', 'NO_PERI_CORR', $
;             'SX_PEAK', 'SX_SOURCE', 'SX_OBJ' ]
;         3: members = $ ; number = 29/32
;           [ '', '', '', 'SX_OBJ', $
;             'SX_SOURCE', 'SX_PEAK', 'MTGL_TAIL', 'MTGL_TYPE4', $
;             'MTGL_TYPE3', 'MTGL_TYPE2', 'MTGL_TYPE1', 'GPGL_TAIL', $
;             'GPGL_TYPE4', 'GPGL_TYPE3', 'GPGL_TYPE2', 'GPGL_TYPE1', $
;             'NO_ABSCAL', 'NO_MTGL', 'NO_GPGL', 'NO_FLAT_FIELD', $
;             'NO_TR_CORR', 'NO_DCCAL_CORR', 'NO_DK_CORR', 'NO_RP_CORR', $
;             'NO_DIFF', 'RSTANOM', 'RESET', 'SATURATE', $
;             'DEAD', 'ARITH_ERR', 'UNDEF_ANOM', 'BAD' ]
;         2: members = $ ; number = 28/32
;           [ '', '', '', '', $
;             '', '', '', 'MTGL_TAIL', $
;             'MTGL_TYPE4', 'MTGL_TYPE3', 'MTGL_TYPE2', 'MTGL_TYPE1', $
;             '', '', '', 'GPGL_TAIL', $
;             'GPGL_TYPE4', 'GPGL_TYPE3', 'GPGL_TYPE2', 'GPGL_TYPE1', $
;             'TRSNTCRR_QL0', 'TRSNTCRR_QL1', 'SPIKE', 'SX_SOURCE', $
;             '', 'BADDARK', 'NOSTIM', 'NODIFF', $
;             'AFTERRST', 'SATURATE', 'DEAD', 'BAD' ]
;         1: members = $ ; number = 12/16
;           [ '', '', '', '', $
;             'TRSNT_CRR_QUALITY', 'NO_DIF', 'AFT_RESET', 'CRSTLK_CRR', $
;             'DARK_BAD', 'NO_STIM', 'SOURCE', 'SPIKE',  $
;             'TRSNT_CRR', 'SATURATE', 'BAD', 'DEAD' ]
;         ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
;     ENDCASE
;     RETURN, members
; END

; FUNCTION fis_obs::Get_Quality_Members
;     COMPILE_OPT hidden
;     IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;     CASE self.extver OF
;         4: BEGIN
;             n = 18 ; number of members
;             members = REPLICATE({s0, name:'', bits:0}, n)
;             members = $
;               [ {s0,'QUAL_CV_PARAM',2},  {s0,'QUAL_RC_PARAM',2}, $
;                 {s0,'QUAL_RC_CF',2},     {s0,'QUAL_DF_EQ',2}, $
;                 {s0,'QUAL_RP_DATA',2},   {s0,'QUAL_RP_PARAM',2}, $
;                 {s0,'QUAL_RP_TABLE',2},  {s0,'QUAL_FF_PARAM',2}, $
;                 {s0,'QUAL_FF_CF',2},     {s0,'QUAL_GPGL_CORR',2}, $
;                 {s0,'QUAL_MTGL_CORR',2}, {s0,'QUAL_TR_HIST',2}, $
;                 {s0,'QUAL_TR_PARAM',2},  {s0,'QUAL_DK_DATA',2}, $
;                 {s0,'QUAL_DK_PARAM',2},  {s0,'QUAL_DK_TABLE',2}, $
;                 {s0,'QUAL_FX_CORR',2},   {s0,'QUAL_FX_PARAM',2} ]
;         END
;         3: BEGIN
;             n = 20 ; number of members
;             members = REPLICATE({s0, name:'', bits:0}, n)
;             members = $
;               [ {s0,'',2},  {s0,'',2}, $
;                 {s0,'QUAL_FX_PARAM',2},  {s0,'QUAL_FX_CORR',2}, $
;                 {s0,'QUAL_DK_TABLE',2},  {s0,'QUAL_DK_PARAM',2}, $
;                 {s0,'QUAL_DK_DATA',2},   {s0,'QUAL_TR_PARAM',2}, $
;                 {s0,'QUAL_TR_HIST',2},   {s0,'QUAL_MTGL_CORR',2}, $
;                 {s0,'QUAL_GPGL_CORR',2}, {s0,'QUAL_FF_CF',2}, $
;                 {s0,'QUAL_FF_PARAM',2},  {s0,'QUAL_RP_TABLE',2}, $
;                 {s0,'QUAL_RP_PARAM',2},  {s0,'QUAL_RP_DATA',2}, $
;                 {s0,'QUAL_DF_EQ',2},     {s0,'QUAL_RC_CF',2}, $
;                 {s0,'QUAL_RC_PARAM',2},  {s0,'QUAL_CV_PARAM',2} ]
;         END
;         2: members = [ {s0,'',0} ]
;         1: members = [ {s0,'',0} ]
;         ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
;     ENDCASE
;     RETURN, members
; END


; INIT should return a scalar TRUE value if the initialization
; is successful, and FALSE if the initialization fails.
FUNCTION fis_obs::INIT, datatype, EXTVER=extver
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF NOT KEYWORD_SET(datatype) THEN BEGIN
        MESSAGE, "Usage: fis = OBJ_NEW('fis_obs', datatype)"
        RETURN, !FALSE
    ENDIF
    self.datatype = ''
    IF KEYWORD_SET(datatype) THEN self.datatype = datatype
    CASE STRUPCASE(STRTRIM(datatype,2)) OF
        'FIS_SW': self.ndet = 100
        'FIS_LW': self.ndet = 75
        'ISO':    self.ndet = 4
        ELSE: MESSAGE, 'Unsupported datatype: "' + datatype + '"'
    ENDCASE
    RETURN, self->bte::INIT('FIS_OBS', EXTVER=extver)
END

PRO fis_obs::CLEANUP
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    self->bte::CLEANUP
END

;----------------------------------------------------------------------------
;Routine to define the class structure
;
PRO fis_obs__define
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    STRUCT_HIDE, { fis_obs, $   ; structure name
          INHERITS bte }        ; inheritance

END
