; $Id: primary__define.pro,v 1.12 2007/01/07 05:06:13 cyamauch Exp $
;
; Copyright(C) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; CLASS_NAME:
;   Primary - Primary HDU of FITS, inherited from HDU
;
; PURPOSE:
;   Define the structure for Primary HDU, which is a part of TSD.
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   None
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   None
;
; DESCRIPTION:
;   See 'ASTRO-F FIS/Survey Data Structure' in detail.
;
; EXAMPLE:
;   None
;
; SIDE EFFECTS:
;   None
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, June 2002.
;
;-

FUNCTION primary::read, lun
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    LMESSAGE, 7, "Reading Primary HDU...", /PRINT
    FXHREAD, lun, prihdr, status
    self.header = PTR_NEW(prihdr)
    self.data = PTR_NEW()

    fmttype = self->Get_Header_Keyword('FMTTYPE')
    IF STRMATCH(STRTRIM(STRING(fmttype),2), '0') THEN BEGIN
        datatype = self->Get_Header_Keyword('DATATYPE')
    ENDIF ELSE BEGIN
        s = STRTRIM(fmttype,2)
        datatype = '';
        IF STRMATCH(s,'ASTRO-F TSD FIS_SW') THEN datatype = 'FIS_SW'
        IF STRMATCH(s,'ASTRO-F TSD FIS_LW') THEN datatype = 'FIS_LW'
        IF STRMATCH(s,'ASTRO-F TSD ISO') THEN datatype = 'ISO'
    ENDELSE
;    DPRINT, 'DATATYPE = ' + datatype
    CASE STRUPCASE(STRTRIM(datatype,2)) OF
        'FIS_SW': fis_ndet = '100'
        'FIS_LW': fis_ndet = '75'
        'ISO': iso_ndet = '4'
        ELSE: MESSAGE, 'Unsupported datatype: "' + datatype + '"'
    ENDCASE

    RETURN, !SUCCEED
END

; This method should be overrided.
FUNCTION primary::write, outfile
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    LMESSAGE, 7, "Writing Primary HDU...", /PRINT

    FXWRITE, outfile, *self.header ;, COMPRESS=compress

    RETURN, !SUCCEED
END

; This method should be overrided.
FUNCTION primary::create, datatype, naxis2, INSTRUME=instrume
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    DPRINT, "Creating Primary HDU..."

    GET_DATE, current_date, /TIMETAG
    IF NOT KEYWORD_SET(instrume) THEN instrume='FIS'

    ; initialize header only
    FXHMAKE, header, /INITIALIZE, /EXTEND, /DATE
    FXADDPAR, header, 'FMTTYPE',  'ASTRO-F TSD ' + datatype, ' Type of Format in FITS file'
    FXADDPAR, header, 'FTYPEVER', self.extver, ' Version of FMTTYPE'
    FXADDPAR, header, 'CNTTYPE',  datatype, ' Type of data content'
    FXADDPAR, header, 'DATE',     current_date, ' File Creation Date'
    FXADDPAR, header, 'CREATOR',  'TSD I/F ', ' Data generator program name'
    FXADDPAR, header, 'CRTRVER',  '4.0.1   ', ' Version of CREATOR'
    FXADDPAR, header, 'PIPELINE', '        ', ' Data Processing Pipeline name'
    FXADDPAR, header, 'DATASTAT', 'GOOD    ', ' Data status'
    FXADDPAR, header, 'ORIGIN',   'JAXA/ISAS', ' Organization creating FITS file'
    FXADDPAR, header, 'TELESCOP', 'AKARI   ', ' AKARI mission'
    FXADDPAR, header, 'INSTRUME', instrume, ' Identifier of the instrument'
    FXADDPAR, header, 'DETECTOR', 'LW      ', ' Detector name'
    FXADDPAR, header, 'OBSERVER', '        ', " PI Name (Observer's ID)"
    FXADDPAR, header, 'PROPOSAL', '        ', ' Proposal ID'
    FXADDPAR, header, 'OBS-CAT',  '        ', ' Observation Category'
    FXADDPAR, header, 'PNTNG-ID', '        ', ' Pointing ID'
    FXADDPAR, header, 'TARGETID', '        ', ' Target ID'
    FXADDPAR, header, 'SUBID',    '        ', ' Sub ID'
    FXADDPAR, header, 'OBJECT',   '        ', ' Object name'
    FXADDPAR, header, 'OBJ-RA',   '        ', ' [degree] Target position'
    FXADDPAR, header, 'OBJ-DEC',  '        ', ' [degree] Target position'
    FXADDPAR, header, 'AOT',      '        ', ' Observation AOT'
    FXADDPAR, header, 'AOTPARAM', '        ', ' AOT Parameter'
    FXADDPAR, header, 'INSTMODE', '        ', ' Instrument operation mode'
    FXADDPAR, header, 'TIMESYS',  'UTC     ', ' Explicit time scale specification'
    FXADDPAR, header, 'DATE-OBS', '        ', ' Observation start date+time'
    FXADDPAR, header, 'DATE-END', '        ', ' Observation end date+time'
    FXADDPAR, header, 'DATE-REF', '        ', ' Reference time in the Observation'
    FXADDPAR, header, 'AFTM-OBS', 0.0, ' DATE-OBS in ASTRO-F Time'
    FXADDPAR, header, 'AFTM-END', 0.0, ' DATE-END in ASTRO-F Time'
    FXADDPAR, header, 'AFTM-REF', 0.0, ' DATE-REF in ASTRO-F Time'
    FXADDPAR, header, 'PIMTIOBS', '        ', ' DATE-OBS in PIM-TI (36bits DHUTI)'
    FXADDPAR, header, 'PIMTIEND', '        ', ' DATE-END in PIM-TI (36bits DHUTI)'
    FXADDPAR, header, 'PIMTIREF', '        ', ' DATE-REF in PIM-TI (36bits DHUTI)'
    FXADDPAR, header, 'EQUINOX', 2000.0000, ' Epoch of Coordinate'
    FXADDPAR, header, 'RA',       0.0, ' [degree] Target position at DATE-REF'
    FXADDPAR, header, 'DEC',      0.0, ' [degree] Target position at DATE-REF'
    FXADDPAR, header, 'ROLL',     0.0, ' [degree] Roll angle at DATE-REF'
    FXADDPAR, header, 'AA-SOL',   0.0, ' [degree] Solar avoidance angle at DATE-REF'
    FXADDPAR, header, 'AA-EAR',   0.0, ' [degree] Earth avoidance angle at DATE-REF'
    FXADDPAR, header, 'AA-LUN',   0.0, ' [degree] Lunar avoidance angle at DATE-REF'
    FXADDPAR, header, 'TM-SAA',   0.0, ' [sec] Duration since last SAA at DATE-REF'
    FXADDPAR, header, 'SAT-POSX', 0.0, ' [km] Satellite position at DATE-REF'
    FXADDPAR, header, 'SAT-POSY', 0.0, ' [km] Satellite position at DATE-REF'
    FXADDPAR, header, 'SAT-POSZ', 0.0, ' [km] Satellite position at DATE-REF'
    FXADDPAR, header, 'DAYNIGHT', '        ', ' day/night status at DATE-REF'
    FXADDPAR, header, 'STTA-NUM', 0, ' number of tracked stars in STT-A at DATE-REF'
    FXADDPAR, header, 'STTB-NUM', 0, ' number of tracked stars in STT-B at DATE-REF'
    FXADDPAR, header, 'STTA-MOD', '        ', ' STT-A Mode status at DATE-REF'
    FXADDPAR, header, 'STTB-MOD', '        ', ' STT-B Mode status at DATE-REF'
    self.header = PTR_NEW(header)
    self.data = PTR_NEW()

    RETURN, !SUCCEED
END


; INIT should return a scalar TRUE value if the initialization
; is successful, and FALSE if the initialization fails.
FUNCTION primary::INIT
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    self.exttype = "PRIMARY"
    self.extname = "PRIMARY"
    self.extver  = 4
    RETURN, self->hdu::INIT()
END
PRO primary::CLEANUP
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    self->hdu::CLEANUP
END


; primary HDU
PRO primary__define
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    STRUCT_HIDE, { primary, $       ; structure name
          INHERITS hdu }
END
