; $Id: util.pro,v 1.13 2006/04/25 06:34:02 baba Exp $
;
; Copyright(C) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   UTIL
;
; PURPOSE:
;   Define the utility functions, which is a part of TSD definition.
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   None
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   None
;
; DESCRIPTION:
;   None
;
; EXAMPLE:
;   None
;
; SIDE EFFECTS:
;   None
;
; REFERENCE:
;   None
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, September 2002.
;   Modified by: Akiko Kawamura, March 2003.
;                Added Strarr in "StructForm"
;   Modified by: Akiko Kawamura, March 2003.
;                Added procedure 'Fxaddpar_array.pro'
;-


;; debug print
PRO dprint, msg
    COMPILE_OPT hidden
    IF !DEBUG THEN BEGIN
        ;; print to stderr (LUN = -2)
;        PRINTF, !STDERR, "% DPRINT: " + STRING(msg)
        MESSAGE, STRING(msg), /CONTINUE
    ENDIF
END

PRO setDebug
    COMPILE_OPT hidden
    !DEBUG = 1B
    IF NOT !QUIET THEN PRINT, 'Debug mode is ON'
END

PRO unsetDebug
    COMPILE_OPT hidden
    !DEBUG = 0B
    IF NOT !QUIET THEN PRINT, 'Debug mode is OFF'
END

FUNCTION isDebug
    COMPILE_OPT hidden
    RETURN, !DEBUG
END


PRO vprint, msg
    COMPILE_OPT hidden
    IF NOT !QUIET THEN BEGIN
        ;; print to stdout (LUN = -1)
;        PRINTF, !STDOUT, "% VPRINT: ", STRING(msg)
        MESSAGE, STRING(msg), /INFORMATIONAL
    ENDIF
END

; Infomational messages should be printed (0) or suppressed (nonzero).
PRO BeQuiet
    COMPILE_OPT hidden
    !QUIET = 1B
END

PRO BeVerbose
    COMPILE_OPT hidden
    !QUIET = 0B
END

FUNCTION StructForm, val
    COMPILE_OPT hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    ndat = STRMID(val, 0, STRLEN(val)-1) ; except last character
    type = STRMID(val, STRLEN(val)-1, 1) ; last character
    ndatx = STRTRIM((LONG(ndat)+7)/8, 2)
;    IF (LONG(ndat) EQ 1) OR (LONG(ndatx) EQ 1) THEN BEGIN
    IF (LONG(ndat) EQ 1) THEN BEGIN
        CASE type OF
            'L': RETURN, "'N'" ; Logical, 'T', 'F', or 'N' (Null) 
            'X': RETURN, '0B' ; Bits
            'B': RETURN, '0B' ; Unsigned Byte
            'I': RETURN, '0S' ; Short (16bit integer)
            'J': RETURN, '0L' ; Long (32bit integer)
            'K': RETURN, '0LL' ; Long Long (64bit integer)
            'E': RETURN, '0.0E' ; Float
            'D': RETURN, '0.0D' ; Double
            'A': RETURN, "''"   ; Character
            'C': RETURN, 'COMPLEX(0,0)'  ; Complex
            'M': RETURN, 'DCOMPLEX(0,0)' ; DComplex
            'S': RETURN, 'StrArr(1)'
            ELSE: MESSAGE, 'Unsupported format: ' + type
        ENDCASE
    ENDIF ELSE BEGIN
        CASE type OF
            'L': RETURN, 'STRArr(' + ndat + ')'
            'X': RETURN, 'BytArr(' + ndatx + ')'
            'B': RETURN, 'BytArr(' + ndat + ')'
            'I': RETURN, 'IntArr(' + ndat + ')'
            'J': RETURN, 'LonArr(' + ndat + ')'
            'K': RETURN, 'Lon64Arr(' + ndat + ')'
            'E': RETURN, 'FltArr(' + ndat + ')'
            'D': RETURN, 'DblArr(' + ndat + ')'
            'A': RETURN, "''"   ; note!
            'C': RETURN, 'ComplexArr(' + ndat + ')'
            'M': RETURN, 'DComplexArr(' + ndat + ')'
            'S': RETURN, 'StrArr(' + ndat + ')'
            ELSE: MESSAGE, 'Unsupported format: ' + type
        ENDCASE
    ENDELSE
    RETURN, !FAIL
END

; psuedo_tmpnam - create a name for a temporary file
FUNCTION psuedo_tmpnam
    COMPILE_OPT hidden
    chars = ['a','b','c','d','e','f','g','h','i','j','k','l','m',$
             'n','o','p','q','r','s','t','u','v','w','x','y','z',$
             'A','B','C','D','E','F','G','H','I','J','K','L','M',$
             'N','O','P','Q','R','S','T','U','V','W','X','Y','Z']
    nStr = 10
    seed = SYSTIME(/SECONDS)
    index = FIX(RANDOMU(seed, nStr) * N_ELEMENTS(chars))
    RETURN, STRJOIN(chars[index], '')
END

PRO FXADDPAR_ARRAY, HEADER, NAME, VALUE, COMMENT, _EXTRA = e
    COMPILE_OPT hidden
; PURPOSE:
;     Procedure to append array to FITS header 
; COMMENT:
;     Call FXADDPAR.PRO in astrolib. 
; MODIFICATION HISTORY:
;     May 2003, Added by A. Kawamura

si = size(VALUE)
IF si[0] EQ 0 then begin
    FXADDPAR, HEADER, NAME, VALUE, COMMENT, _STRICT_EXTRA = e
ENDIF ELSE BEGIN
    FOR I = 0, si[1] - 1 DO BEGIN
        IF VALUE[i] NE '' THEN FXADDPAR, HEADER, NAME, VALUE[I], COMMENT, _STRICT_EXTRA=e
    ENDFOR
ENDELSE

END

;----------------------------------------------------------------------------
;
PRO util
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    ; Define some system variables.
    ; This part can not move to util::INIT procedure...
    !QUIET = 1B                 ; Be quiet if !QUIET is non zero.
    DEFSYSV, '!DEBUG', EXISTS = e
    IF NOT e THEN DEFSYSV, '!DEBUG', 0B
    DEFSYSV, '!SUCCEED', EXISTS = e
    IF NOT e THEN DEFSYSV, '!SUCCEED', 0S, 1 ; Read_Only
    DEFSYSV, '!FAIL', EXISTS = e
    IF NOT e THEN DEFSYSV, '!FAIL', -1S, 1 ; Read_Only
    DEFSYSV, '!TRUE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!TRUE', 1B, 1 ; Read_Only
    DEFSYSV, '!FALSE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!FALSE', 0B, 1 ; Read_Only
    DEFSYSV, '!IDL_TRUE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!IDL_TRUE', 1B, 1 ; Read_Only
    DEFSYSV, '!IDL_FALSE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!IDL_FALSE', 0B, 1 ; Read_Only

    DEFSYSV, '!STDIN', EXISTS = e
    IF NOT e THEN DEFSYSV, '!STDIN', 0S, 1 ; Read_Only
    DEFSYSV, '!STDOUT', EXISTS = e
    IF NOT e THEN DEFSYSV, '!STDOUT', -1S, 1 ; Read_Only
    DEFSYSV, '!STDERR', EXISTS = e
    IF NOT e THEN DEFSYSV, '!STDERR', -2S, 1 ; Read_Only

    DEFSYSV, '!LDS_URL', EXISTS = e
;    IF NOT e THEN DEFSYSV, '!LDS_URL', 'http://afsvr02.ir.isas.jaxa.jp/LDS/' ; NOT Read_Only
    IF NOT e THEN DEFSYSV, '!LDS_URL', 'http://www.ir.isas.jaxa.jp/AKARI/iris_data/LDS/' ; NOT Read_Only

END

;----------------------------------------------------------------------------

